
//=============================================================================
//
//      cpu_gpio.h
//
//      CPU/Board Specific GPIO assignments
//
// Author(s):    Michael Kelly, Cogent Computer Systems, Inc.
// Date:         09/18/2005
// Description:  This file contains the GPIOIO usage for the CSB535FS
//
//
//=============================================================================

// PORT A - Virtual GPIO Bits 0-31
#define PORTA_VBIT      0
// Signal			    Bit		               Dir	Active	Usage
#define	LCD_VEE_BIT	    BIT30	            // Out  High    LCD VEE Enable
#define	LCD_VEE	        PORTA_VBIT + 30
#define	CF_VEN_BIT	    BIT27	            // Out  Low     Compact Flash Power Enable
#define	CF_VEN	        PORTA_VBIT + 27
#define	CF_RDY_BIT	    BIT26	            // In   n/a     Compact Flash Ready/Busy
#define	CF_RDY	        PORTA_VBIT + 26
#define	CF_RST_BIT	    BIT25	            // Out  High    Compact Flash Reset
#define	CF_RST	        PORTA_VBIT + 25
#define	CF_CD_BIT	    BIT24	            // In   Low     Compact Flash Card Detect
#define	CF_CD	        PORTA_VBIT + 24
#define	USR_SW1_BIT		BIT18	        	// In	Low		User Switch 1 on CPU Module
#define	USR_SW1		    PORTA_VBIT + 18
#define	USR_SW0_BIT		BIT6	        	// In	Low		User Switch 0 on CPU Module
#define	USR_SW0		    PORTA_VBIT + 6

// Define initial direction (1 = output) and value
#define PORTA_DDIR      (LCD_VEE_BIT | CF_RST_BIT | CF_VEN_BIT)
#define PORTA_DR        (CF_RST_BIT)
#define PORTA_OCR1      0xffffffff      // all PORTA GPIO's but PA17 are driven by the GPIO Data Register
#define PORTA_OCR2      0xffffffff      // all PORTA GPIO's but PA17 are driven by the GPIO Data Register
#define PORTA_ICR1      0x00000000      // all PORTA GPIO's inputs but PA17 go to the GPIO Sample Status Register
#define PORTA_ICR2      0x00000000      // all PORTA GPIO's inputs but PA17 go to the GPIO Sample Status Register
       
// Define signals that are to be set as GPIO (bit = 1) and define for Primary (0)
// or Secondary (1) use non-GPIO
#define PORTA_GPIUS		(LCD_VEE_BIT | CF_CD_BIT | CF_RST_BIT | CF_RDY_BIT | CF_VEN_BIT | USR_SW1_BIT | USR_SW0_BIT)
#define PORTA_GPR       0x0 // no secondary functions

// -----------------------------------------------------------------------------------------------
// PORT B
#define PORTB_VBIT      32
// Signal			    Bit		               Dir	Active	Usage

// Define initial direction (1 = output) and value
#define PORTB_DDIR      0
#define PORTB_DR        0
#define PORTB_OCR1      0xffffffff      // all PORTB GPIO's are driven by the GPIO Data Register
#define PORTB_OCR2      0xffffffff      // all PORTB GPIO's are driven by the GPIO Data Register
#define PORTB_ICR1      0x0             // all PORTB GPIO's inputs go to the GPIO Sample Status Register
#define PORTB_ICR2      0x0             // all PORTB GPIO's inputs go to the GPIO Sample Status Register
       
// Define signals that are to be set as GPIO (bit = 1) and define for Primary (0)
// or Secondary (1) use non-GPIO
#define PORTB_GPIUS     0x0
#define PORTB_GPR       0x0 // no secondary functions

// -----------------------------------------------------------------------------------------------
// PORT C
#define PORTC_VBIT      64

// Signal			    	Bit		               Dir	Active	Usage
#define	LCD_VEN_BIT	    	BIT27	            // Out  High    LCD Power Enable
#define	LCD_VEN	        	PORTC_VBIT + 27
#define	USR_LED1_BIT 		BIT15	            // Out	Low    User LED 1
#define	USR_LED1	     	PORTC_VBIT + 15
#define	USR_LED0_BIT 		BIT14	            // Out	Low    User LED 0
#define	USR_LED0	     	PORTC_VBIT + 14
//#define	USBD_VP_BIT 	BIT13	            // Out	High    USB VP
//#define	USBD_VP     	PORTC_VBIT + 13
//#define	USBD_VM_BIT 	BIT12	            // Out	High    USB VM
//#define	USBD_VM     	PORTC_VBIT + 12
//#define	USBD_VPO_BIT 	BIT11	            // Out	High    USB VPO
//#define	USBD_VPO     	PORTC_VBIT + 11
//#define	USBD_VMO_BIT 	BIT10	            // Out	High    USB VMO
//#define	USBD_VMO     	PORTC_VBIT + 10
//#define	USBD_OE_BIT 	BIT9	            // Out	Low    USB Output Enable
//#define	USBD_OE     	PORTC_VBIT + 9
//#define	USBD_SPEED_BIT 	BIT8	            // Out	High    USB Speed
//#define	USBD_SPEED     	PORTC_VBIT + 8
//#define	USBD_SUSP_BIT   BIT7	            // Out	High    USB Suspend
//#define	USB_SUSP        PORTC_VBIT + 7
#define	USBD_MODE_BIT   BIT6	            // Out  High		USB Mode
#define	USBD_MODE       PORTC_VBIT + 6
#define	USBD_EN_BIT	    BIT5	            // Out  High    USBD Enable
#define	USBD_EN	        PORTC_VBIT + 5

// Define initial direction (1 = output) and value
#define PORTC_DDIR      (LCD_VEN_BIT | USBD_EN_BIT | USBD_MODE_BIT | USR_LED0_BIT | USR_LED1_BIT)
#define PORTC_DR 		(USR_LED0_BIT  | USR_LED1_BIT)
#define PORTC_OCR1      0xffffffff      // all PORTC GPIO's are driven by the GPIO Data Register
#define PORTC_OCR2      0xffffffff      // all PORTC GPIO's are driven by the GPIO Data Register
#define PORTC_ICR1      0x0             // all PORTC GPIO's inputs go to the GPIO Sample Status Register
#define PORTC_ICR2      0x0             // all PORTC GPIO's inputs go to the GPIO Sample Status Register
       
// Define signals that are to be set as GPIO (bit = 1) and define for Primary (0)
// or Secondary (1) use non-GPIO
#define PORTC_GPIUS		(LCD_VEN_BIT | USBD_EN_BIT | USBD_MODE_BIT | USR_LED0_BIT | USR_LED1_BIT)
#define PORTC_GPR       0x0 // no secondary functions

// -----------------------------------------------------------------------------------------------
// PORT D
#define PORTD_VBIT      96

// Signal			    Bit		               Dir	Active	Usage
#define	TSEN_BIT  	  	BIT28	            // Out	High    USB Device Soft Connect, 1 = Device Present
#define	TSEN        	PORTD_VBIT + 28
#define	SD_WP_BIT	    BIT27	            // In	High	SD Card Write Protect Switch
#define	SD_WP	        PORTD_VBIT + 27
#define	SD_CD_BIT	    BIT26	            // In	Low		SD Card Detect
#define	SD_CD	        PORTD_VBIT + 26
#define	SD_VEN_BIT	    BIT25	            // In  Low		SD Card Detect
#define	SD_VEN	        PORTD_VBIT + 25
#define	I2C_SDA_BIT		BIT15	        	// I/O	NA		I2C Data
#define	I2C_SDA		    PORTA_VBIT + 18
#define	I2C_SCL_BIT		BIT16	        	// I/O	NA		I2C Clock
#define	I2C_SCL		    PORTA_VBIT + 17

// Define initial direction (1 = output) and value
#define PORTD_DDIR		TSEN_BIT            
#define PORTD_DR        TSEN_BIT
#define PORTD_OCR1      0xffffffff      // all PORTD GPIO's are driven by the GPIO Data Register
#define PORTD_OCR2      0xffffffff      // all PORTD GPIO's are driven by the GPIO Data Register
#define PORTD_ICR1      0x0             // all PORTD GPIO's inputs go to the GPIO Sample Status Register
#define PORTD_ICR2      0x0             // all PORTD GPIO's inputs go to the GPIO Sample Status Register

       
// Define signals that are to be set as GPIO (bit = 1) and define for Primary (0)
// or Secondary (1) use non-GPIO
#define PORTD_GPIUS		(SD_VEN_BIT | SD_CD_BIT | SD_WP_BIT | TSEN_BIT)
#define PORTD_GPR       0x0 // no secondary functions

// -----------------------------------------------------------------------------------------------
// PORT E
#define PORTE_VBIT      128

// Signal			    Bit		               Dir	Active	Usage
#define USB_SCON_BIT	BIT15				// Out	High	USB Device Soft Connect, 1 = Device Present 
#define	USB_SCON		PORTE_VBIT + 15
#define USB_CD_BIT		BIT14				// In	High	USB Cable Detect
#define	USB_CD			PORTE_VBIT + 14
#define	LCD_BKL_BIT		BIT5	        	// Out	High	LCD Backlight Enable (PWMO)
#define	LCD_BKL		    PORTE_VBIT + 5
#define	E_INT_BIT	    BIT2	            // In	High	CS8900a Interrupt
#define	E_INT	        PORTE_VBIT + 2
#define	PIRQ_BIT	    BIT1	            // In	Low     ADS7846 Touch Interrupt
#define	PIRQ	        PORTE_VBIT + 1
#define	I2C_INT_BIT	    BIT0	            // In	Low     I2C Expander Interrupt
#define	I2C_INT	        PORTE_VBIT + 0

// Define initial direction (1 = output) and value
#define PORTE_DDIR      (USB_SCON_BIT | LCD_BKL_BIT)    
#define PORTE_DR        0
#define PORTE_OCR1      0xffffffff      // all PORTD GPIO's are driven by the GPIO Data Register
#define PORTE_OCR2      0xffffffff      // all PORTD GPIO's are driven by the GPIO Data Register
#define PORTE_ICR1      0x0             // all PORTD GPIO's inputs go to the GPIO Sample Status Register
#define PORTE_ICR2      0x0             // all PORTD GPIO's inputs go to the GPIO Sample Status Register

       
// Define signals that are to be set as GPIO (bit = 1) and define for Primary (0)
// or Secondary (1) use non-GPIO
#define PORTE_GPIUS     (LCD_BKL_BIT | E_INT_BIT | PIRQ_BIT | I2C_INT_BIT | USB_CD_BIT | USB_SCON_BIT)
#define PORTE_GPR       0x0 // no secondary functions

// -----------------------------------------------------------------------------------------------
// PORT F
#define PORTF_VBIT      160

// Signal			    Bit		               Dir	Active	Usage

// Define initial direction (1 = output) and value
#define PORTF_DDIR      0      
#define PORTF_DR        0
#define PORTF_OCR1      0xffffffff      // all PORTD GPIO's are driven by the GPIO Data Register
#define PORTF_OCR2      0xffffffff      // all PORTD GPIO's are driven by the GPIO Data Register
#define PORTF_ICR1      0x0             // all PORTD GPIO's inputs go to the GPIO Sample Status Register
#define PORTF_ICR2      0x0             // all PORTD GPIO's inputs go to the GPIO Sample Status Register

       
// Define signals that are to be set as GPIO (bit = 1) and define for Primary (0)
// or Secondary (1) use non-GPIO
#define PORTF_GPIUS		0
#define PORTF_GPR       0x0 // no secondary functions
