/* coldstart.s:
 * First bit of boot code run by the processor.
 *
 * General notice:
 * This code is part of a boot-monitor package developed as a generic base
 * platform for embedded system designs.  As such, it is likely to be
 * distributed to various projects beyond the control of the original
 * author.  Please notify the author of any enhancements made or bugs found
 * so that all may benefit from the changes.  In addition, notification back
 * to the author will allow the new user to pick up changes that may have
 * been made by other users after this version of the code was distributed.
 *
 * Author:  Ed Sutter
 * email:   esutter@lucent.com 
 * phone:   908-582-2351
 *
 * This code was derived from source code from Motorola...
 *
 *********************************************************************
 *
 * Copyright:
 *  MOTOROLA, INC. All Rights Reserved.  
 *  You are hereby granted a copyright license to use, modify, and
 *  distribute the SOFTWARE so long as this entire notice is
 *  retained without alteration in any modified and/or redistributed
 *  versions, and that such modified versions are clearly identified
 *  as such. No licenses are granted by implication, estoppel or
 *  otherwise under any patents or trademarks of Motorola, Inc. This 
 *  software is provided on an "AS IS" basis and without warranty.
 *
 *  To the maximum extent permitted by applicable law, MOTOROLA 
 *  DISCLAIMS ALL WARRANTIES WHETHER EXPRESS OR IMPLIED, INCLUDING 
 *  IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR
 *  PURPOSE AND ANY WARRANTY AGAINST INFRINGEMENT WITH REGARD TO THE 
 *  SOFTWARE (INCLUDING ANY MODIFIED VERSIONS THEREOF) AND ANY 
 *  ACCOMPANYING WRITTEN MATERIALS.
 * 
 *  To the maximum extent permitted by applicable law, IN NO EVENT
 *  SHALL MOTOROLA BE LIABLE FOR ANY DAMAGES WHATSOEVER (INCLUDING 
 *  WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS 
 *  INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY
 *  LOSS) ARISING OF THE USE OR INABILITY TO USE THE SOFTWARE.   
 * 
 *  Motorola assumes no responsibility for the maintenance and support
 *  of this software
 ********************************************************************
 */
    .file   "coldstart.S"

#include "cpuio.h"
#include "warmstart.h"
#include "mcf5272.h"
    
#define STACKsize 2048
#define STACKtop (STACKsize-32)
#define COLDSTART_OFFSET 0x80
#define INITIAL_TOP_OF_STACK	(INT_SRAM_ADDRESS+INT_SRAM_SIZE-32)

    .global resetSP, resetPC
    .global coldstart
    .global ram_coldstart
    .extern continue

    .text

/* The reset point:
 *
 * The flash chip select is tied to CS0, and at powerup or reset, the
 * address at which CS0 is active is zero.  As a result, the long value
 * used as the initial PC must be the offset from zero at which coldstart
 * is located.  We cannot put ".long coldstart" at the resetPC location
 * because it would be populated with the 0xff80XXXX value that it was
 * linked to and that would cause the CPU to deassert CS0 immediately.
 */
resetSP:	/* Initial SP = end of internal SRAM */
    .long   INITIAL_TOP_OF_STACK

resetPC:	/* Initial PC */
    .long   COLDSTART_OFFSET

/* Note: if we were running with a flash-based vector table, then
 * it would reside here.  This version of the monitor builds its
 * vector table into RAM (and sets VBR accordingly) when vinit()
 * is called; hence, for a brief amount of time at bootup the
 * cpu is running without the ability to deal with exceptions
 * (other than RESET).
 */

#include "moncomptr.S"
#include "etheraddr.S"

/* At power-on reset, the initial PC must be between 0-4K so that 
 * CS0 will assert for accesses to Flash.  At this point, we need to
 * clear the CS0 address mask and change the PC to the real Flash 
 * location of 0xFF80XXXX.  This is why coldstart.s is separate from
 * reset.s.  The fact that the "continue" tag is in another file 
 * forces the "jmp continue" instruction below to use the absolute
 * address of "continue".  This sets the PC up for 0xff800000-relative
 * address space.
 * Use .balign here so that the location of coldstart is independent
 * (somewhat) of the code/data placed between reset and coldstart.
 */
	.balign	COLDSTART_OFFSET

coldstart:
    move.l      #IMM_ADDRESS+1,%d0	/* Load MBAR with validate bit set */
    movec       %d0,%MBAR
    move.l      #IMM_ADDRESS,%a0	/* Set CS0 wait states. */
    move.l      #0x20,%d0
    move.l      %d0,MCF5272_CS_CSOR0(%a0)
ram_coldstart:
    move.l      #INITIALIZE,%d5
    jmp         continue
