//=============================================================================
//
//      cpuio.c
//
//      CPU/Board Specific IO
//
// Author(s):    Michael Kelly, Cogent Computer Systems, Inc.
// Date:         05-26-2002
// Description:  This file contains the IO functions required by Micro Monitor
//				 that are unique to each CPU/Board combination
//
//
// Modified by Technology Rescue.
//	21.03.2005 : Debugging cf_init.
//	22.03.2005 : Moved some inits from devInit to initCPUio.
//
//=============================================================================

#include "config.h"
#include "cpuio.h"
#include "genlib.h"
#include "ether.h"
#include "stddefs.h"
#include "warmstart.h"
#include "misc_arm.h"
#include "uart16550.h"
#include "pxa250.h"
#include "pxa250_lcd.h"
#include "pxa250_gpio.h"

extern ulong i2c_init();

// Delay for some usecs. - Not accurate, assumes ROM mode
// and no Cache
void udelay(int delay)
{
  int i;
  for ( i = LOOPS_PER_USEC * delay; i ; i--)
	;
}

void
cf_init(void)
//{
	// read the CD signals to see if we have a card
	// both must be 0
//	if (GPIO_tst(12)){
//		printf("No Compact FLASH Card Detected.\n");
//		return;
//	}

//	printf("Compact FLASH Card Detected.\n");

//	GPIO_set(13);		// reset
//	udelay(100);
//	GPIO_set(11);		//enable power
//	udelay(100);
//	GPIO_clr(13);		// release reset
//	udelay(100);
//}

{
	// read the CD signals to see if we have a card
	// both must be 0
	if (GPIO_tst(12)){
#ifdef CF_DETECT
		printf("No Compact FLASH Card Detected.\n");
#endif
		return;
	}

#ifdef CF_DETECT
	printf("Compact FLASH Card Detected...");
#endif

	// setup the PCMCIA/CF timing registers for maximum delay
	// this is necessary until the card CIS can be read and
	// actual operating parameters determined
	MSC_REG(MSC_MCMEM0) = 0x000fcfff;
	MSC_REG(MSC_MCMEM1) = 0x000fcfff;
	MSC_REG(MSC_MCATT0) = 0x000fcfff;
	MSC_REG(MSC_MCATT1) = 0x000fcfff;
	MSC_REG(MSC_MCIO0) = 0x000fcfff;
	MSC_REG(MSC_MCIO1) = 0x000fcfff;

	MSC_REG(MSC_MECR) = MECR_CIT		// Card is inserted
					  |	MECR_NOS;		// Number of sockets is 2

	// enable power to the card - 3.3V ONLY!
	GPIO_set(11);

	// delay
	udelay(10000);

	//reset the compact flash
	 GPIO_set(13);

	// delay
	udelay(1000);

	//release reset
	 GPIO_clr(13);

//	if (GPIO_tst(12) == 0){
//		GPIO_set(11); // turn off the power!
//		printf("Power Error, Removing Card Power!\n");
//		return;
//	}
#ifdef CF_DEBUG
	printf("Power OK!\n");
#endif
}

int
getUartDivisor(int baud, unsigned char *hi, unsigned char *lo)
{
	*lo = ((14745600/16)/baud) & 0x00ff;
	*hi = (((14745600/16)/baud) & 0xff00) >> 8;
	return(0);
}

int
devInit(int baud)
{
	lcd_color_depth = 16;

	// initialize the GPIO
	GPIO_init();

	// Initialize the UART
    InitUART(baud);

	//Turn on 5 Volt Power
	GPIO_set(35);

	// Initialize the I2C Port
	i2c_init();

	// Initialize the Compact Flash
	cf_init();

	// INitialize the LCD controller
	lcd_init();
	lcd_on();

	// Turn on User LED
	GPIO_clr(5);

#if INCLUDE_MMC
	/* Initialize SD/MMC card if any. */
	MmcInit();		// Verbose mode = 0.
#endif

	return 0;
}

void
initCPUio()
{
}

void
target_reset(void)
{
	warmstart(INITIALIZE);
}

void
intsrestore(psr)
ulong     psr;
{
    putpsr(psr);
}

/*
 * Read the program status register (CPSR)
 * and set the FIQ and IRQ bits.
 */
ulong
intsoff()
{
	unsigned long psr;

	psr = getpsr();

    /*
     * Set bit 6, bit 7 to disable interrupts.
     */
    psr |= 0x000000c0;

    putpsr(psr);
    return(psr);
}

/* cacheInitForTarget():
 * Enable instruction cache only...
 */
void
cacheInitForTarget()
{
    asm("   MRC p15, 0, r0, c1, c0, 0");
    asm("   ORR r0, r0, #0x1000");  /* bit 12 is ICACHE enable*/
    asm("   MCR p15, 0, r0, c1, c0, 0");

    /* Flush instruction cache */
    asm("   MCR p15, 0, r0, c7, c5, 0");
}
