
//=============================================================================
//
//      cpu_gpio.h
//
//      CPU/Board Specific GPIO assignments
//
// Author(s):    Michael Kelly, Cogent Computer Systems, Inc.
// Date:         09/18/2005
// Description:  This file contains the GPIOIO usage for the CSB735
//
//
//=============================================================================

// PORT A - Virtual GPIO Bits 0-31
#define PORTA_VBIT      0
// Signal			    Bit		               Dir	Active	Usage
#define	LCD_BKL_BIT	    BIT30	            // Out  High    LCD Backlight Enable
#define	LCD_BKL	        PORTA_VBIT + 30

// Define initial direction (1 = output) and value
#define PORTA_DDIR      (LCD_BKL_BIT)
#define PORTA_DR        0
#define PORTA_OCR1      0xffffffff      // all PORTA GPIO's but PA17 are driven by the GPIO Data Register
#define PORTA_OCR2      0xffffffff      // all PORTA GPIO's but PA17 are driven by the GPIO Data Register
#define PORTA_ICR1      0x00000000      // all PORTA GPIO's inputs but PA17 go to the GPIO Sample Status Register
#define PORTA_ICR2      0x00000000      // all PORTA GPIO's inputs but PA17 go to the GPIO Sample Status Register
       
// Define signals that are to be set as GPIO (bit = 1) and define for Primary (0)
// or Secondary (1) use non-GPIO
#define PORTA_GPIUS		(LCD_BKL_BIT)
#define PORTA_GPR       0x0 // no secondary functions

// -----------------------------------------------------------------------------------------------
// PORT B
#define PORTB_VBIT      32
// Signal			    Bit		               Dir	Active	Usage
#define	GPIO4_BIT	    BIT31	            // In   High    SODIMM GPIO 4
#define	GPIO4	        PORTB_VBIT + 31
#define	GPIO5_BIT	    BIT30	            // In   High    SODIMM GPIO 5
#define	GPIO5		    PORTB_VBIT + 30
#define	D_RXD_BIT	    BIT29	            // Out  High	Debug UART Receive
#define	D_RXD	        PORTB_VBIT + 29
#define	D_TXD_BIT	    BIT28	            // Out  High    Debug UART Transmit
#define	D_TXD	        PORTB_VBIT + 28
#define	UH_MODE_BIT		BIT22	            // Out  High    USB Host Mode signal. Always set to 0
#define	UH_MODE			PORTB_VBIT + 22

// Define initial direction (1 = output) and value
#define PORTB_DDIR		(D_TXD_BIT | UH_MODE_BIT)
#define PORTB_DR        0
#define PORTB_OCR1      0xffffffff      // all PORTB GPIO's are driven by the GPIO Data Register
#define PORTB_OCR2      0xffffffff      // all PORTB GPIO's are driven by the GPIO Data Register
#define PORTB_ICR1      0x0             // all PORTB GPIO's inputs go to the GPIO Sample Status Register
#define PORTB_ICR2      0x0             // all PORTB GPIO's inputs go to the GPIO Sample Status Register

// Define signals that are to be set as GPIO (bit = 1) and define for Primary (0)
// or Secondary (1) use non-GPIO
#define PORTB_GPIUS		(D_RXD_BIT | UH_MODE_BIT | GPIO4_BIT | GPIO5_BIT)
#define PORTB_GPR		(D_TXD_BIT)

// -----------------------------------------------------------------------------------------------
// PORT C
#define PORTC_VBIT      64

// Signal			    Bit		               Dir	Active	Usage
#define	I2C_INT_BIT	    BIT31	            // In	Low		SODIMM I2C Interrupt    
#define	I2C_INT	        PORTC_VBIT + 31
#define	NGPIO_BIT	    BIT30	            // In	High	NAND Flash GPIO    
#define	NGPIO	        PORTC_VBIT + 30
#define	NRDY_BIT	    BIT29	            // In	High	NAND Flash Ready Flag    
#define	NRDY	        PORTC_VBIT + 29
#define	GPIO6_BIT 		BIT19	            // In	High	SODIMM GPIO6
#define	GPIO6	     	PORTC_VBIT + 19
#define	GPIO7_BIT 		BIT18	            // In	High	SODIMM GPIO7
#define	GPIO7	     	PORTC_VBIT + 18
#define	GPIO8_BIT 		BIT17	            // In	High	SODIMM GPIO8
#define	GPIO8	     	PORTC_VBIT + 17
#define	GPIO9_BIT 		BIT16	            // In	High	SODIMM GPIO9
#define	GPIO9	     	PORTC_VBIT + 16
#define	GPIO3_BIT 		BIT15	            // In	High	SODIMM GPIO3 (or MX21 Timer In)    
#define	GPIO3	     	PORTC_VBIT + 15
#define	SSI_MCLK_BIT	BIT14	            // In	N/A		Optional SSI Master Clock    
#define	SSI_MCLK     	PORTC_VBIT + 14
#define	UD_MODE_BIT		BIT6	            // Out  Low		USB Device Mode Signal, always set to 0	
#define	UD_MODE			PORTC_VBIT + 6

// Define initial direction (1 = output) and value
#define PORTC_DDIR      UD_MODE_BIT
#define PORTC_DR 		0
#define PORTC_OCR1      0xffffffff			// all PORTC GPIO's are driven by the GPIO Data Register
#define PORTC_OCR2      0xffffffff			// all PORTC GPIO's are driven by the GPIO Data Register
#define PORTC_ICR1      0x0					// all PORTC GPIO's inputs go to the GPIO Sample Status Register
#define PORTC_ICR2      0x0					// all PORTC GPIO's inputs go to the GPIO Sample Status Register
       
// Define signals that are to be set as GPIO (bit = 1) and define for Primary (0)
// or Secondary (1) use non-GPIO
#define PORTC_GPIUS		(UD_MODE_BIT | GPIO3_BIT | GPIO9_BIT | GPIO8_BIT | GPIO7_BIT | GPIO6_BIT | NGPIO_BIT | NRDY_BIT | I2C_INT_BIT)
#define PORTC_GPR       (SSI_MCLK_BIT)

// -----------------------------------------------------------------------------------------------
// PORT D
#define PORTD_VBIT      96

// Signal			    Bit		               Dir	Active	Usage
#define	TSEN_BIT	    BIT28	            // In	Low		ADS7843 CS tied to SPI1_CS0
#define	TSEN	        PORTD_VBIT + 28
#define	SPI0_RDY_BIT    BIT26	            // In	High	SPI Bus 0 Ready
#define	SPI0_RDY        PORTD_VBIT + 26
#define	USBH_VPO_BIT 	BIT24	            // Out	High    USBH VPO
#define	USBH_VPO     	PORTC_VBIT + 24
#define	USBH_VMO_BIT 	BIT23	            // Out	High    USBH VMO
#define	USBH_VMO     	PORTC_VBIT + 23
#define	USBH_OE_BIT 	BIT22	            // Out	Low		USBH Output Enable
#define	USBH_OE     	PORTC_VBIT + 22
#define	USBH_SPEED_BIT 	BIT21	            // Out	High    USBH Speed
#define	USBH_SPEED     	PORTC_VBIT + 21
#define	USBH_VP_BIT 	BIT20	            // In	High    USBH VP
#define	USBH_VP     	PORTD_VBIT + 20
#define	USBH_VM_BIT 	BIT19	            // In	High    USBH VM
#define	USBH_VM     	PORTD_VBIT + 19
#define I2C_SCL_BIT     BIT18				// I/O	NA		I2C Clock
#define I2C_SCL         PORTD_VBIT + 18
#define I2C_SDA_BIT		BIT17				// I/O	NA		I2C Data
#define I2C_SDA         PORTD_VBIT + 17

// Define initial direction (1 = output) and value
#define PORTD_DDIR		(USBH_VPO_BIT | USBH_VMO_BIT | USBH_OE_BIT | USBH_SPEED_BIT)
#define PORTD_DR        TSEN_BIT
#define PORTD_OCR1      0xffffffff      // all PORTD GPIO's are driven by the GPIO Data Register
#define PORTD_OCR2      0xfffC03ff      // all PORTD GPIO's are driven by the GPIO Data Register
#define PORTD_ICR1      0x0             // all PORTD GPIO's inputs go to the GPIO Sample Status Register
#define PORTD_ICR2      0x0             // all PORTD GPIO's inputs go to the GPIO Sample Status Register

       
// Define signals that are to be set as GPIO (bit = 1) and define for Primary (0)
// or Secondary (1) use non-GPIO
#define PORTD_GPIUS     (TSEN_BIT | USBH_VP_BIT | USBH_VM_BIT | USBH_SPEED_BIT | USBH_OE_BIT | USBH_VMO_BIT | USBH_VPO_BIT | SPI0_RDY_BIT)
#define PORTD_GPR       0x0 // no secondary functions

// -----------------------------------------------------------------------------------------------
// PORT E
#define PORTE_VBIT      128

// Signal			    Bit		               Dir	Active	Usage
#define SPI0_CLK_BIT	BIT23				// Out	High	SPI Bus 1 Clock
#define	SPI0_CLK		PORTE_VBIT + 23
#define SPI0_MOSI_BIT	BIT22				// Out	High	SPI Bus 1 Master Out/Slave In
#define	SPI0_MOSI		PORTE_VBIT + 22
#define SPI0_CS0_BIT	BIT21				// Out	High	SPI Bus 1 Chip Select
#define	SPI0_CS0		PORTE_VBIT + 21
#define GPIO0_BIT		BIT20				// Out	High	SODIMM GPIO0 (User LED)
#define	GPIO0			PORTE_VBIT + 20
#define GPIO1_BIT		BIT19				// Out	High	SODIMM GPIO1
#define	GPIO1			PORTE_VBIT + 19
#define SPI0_MIS0_BIT	BIT18				// In	High	SPI Bus 1 Master In/Slave Out
#define	SPI0_MIS0		PORTE_VBIT + 18
#define	GPIO2_BIT		BIT5	        	// Out	High	SODIMM GPIO2 (or MX21 Timer/PWM Out)
#define	GPIO2		    PORTE_VBIT + 5
#define	E_INT_BIT	    BIT2	            // In	High	LAN9211 Interrupt
#define	E_INT	        PORTE_VBIT + 2
#define	PIRQ_BIT	    BIT1	            // In	Low     Interrupt
#define	PIRQ	        PORTE_VBIT + 1
#define	EXP_INT_BIT	    BIT0	            // In	Low     Expansion Interrupt
#define	EXP_INT	        PORTE_VBIT + 0

// Define initial direction (1 = output) and value
#define PORTE_DDIR      GPIO0_BIT    
#define PORTE_DR        0
#define PORTE_OCR1      0xffffffff      // all PORTD GPIO's are driven by the GPIO Data Register
#define PORTE_OCR2      0xffffffff      // all PORTD GPIO's are driven by the GPIO Data Register
#define PORTE_ICR1      0x0             // all PORTD GPIO's inputs go to the GPIO Sample Status Register
#define PORTE_ICR2      0x0             // all PORTD GPIO's inputs go to the GPIO Sample Status Register

       
// Define signals that are to be set as GPIO (bit = 1) and define for Primary (0)
// or Secondary (1) use non-GPIO
#define PORTE_GPIUS     (E_INT_BIT | PIRQ_BIT | EXP_INT_BIT | GPIO0_BIT | GPIO1_BIT | GPIO2_BIT)
#define PORTE_GPR       (SPI0_MIS0_BIT | SPI0_CS0_BIT | SPI0_MOSI_BIT | SPI0_CLK_BIT)

// -----------------------------------------------------------------------------------------------
// PORT F
#define PORTF_VBIT      160

// Signal			    Bit		               Dir	Active	Usage
#define	LOW_PWR_BIT	    BIT22	            // In   Low     Low Power
#define	LOW_PWR	        PORTF_VBIT + 22
#define	CF_CD_BIT	    BIT13	            // In   Low     Compact Flash Card Detect
#define	CF_CD	        PORTF_VBIT + 13
#define	CF_WAIT_BIT	    BIT12	            // In   Low     Compact Flash Wait
#define	CF_WIAT	        PORTF_VBIT + 12
#define	CF_RDY_BIT	    BIT11	            // In   n/a     Compact Flash Ready/Busy
#define	CF_RDY	        PORTF_VBIT + 11
#define	CF_OE_BIT	    BIT4	            // Out  Low		Compact Flash output Enable
#define	CF_OE	        PORTF_VBIT + 4
#define	CF_CE2_BIT	    BIT2	            // Out  High    Compact Flash Chip Enable 2
#define	CF_CE2	        PORTF_VBIT + 2
#define	CF_CE1_BIT	    BIT1	            // Out  High    Compact Flash Chip Enable 1
#define	CF_CE1	        PORTF_VBIT + 1
#define	CF_RST_BIT	    BIT0	            // Out  High    Compact Flash Reset
#define	CF_RST	        PORTF_VBIT + 0

// Define initial direction (1 = output) and value
#define PORTF_DDIR      (CF_OE_BIT | CF_CE2_BIT | CF_CE1_BIT | CF_RST_BIT)
#define PORTF_DR        0
#define PORTF_OCR1      0xfffffCC0      // all PORTD GPIO's are driven by the GPIO Data Register
#define PORTF_OCR2      0xffffffff      // all PORTD GPIO's are driven by the GPIO Data Register
#define PORTF_ICR1      0x0             // all PORTD GPIO's inputs go to the GPIO Sample Status Register
#define PORTF_ICR2      0x0             // all PORTD GPIO's inputs go to the GPIO Sample Status Register

       
// Define signals that are to be set as GPIO (bit = 1) and define for Primary (0)
// or Secondary (1) use non-GPIO
#define PORTF_GPIUS		(LOW_PWR_BIT | CF_CD_BIT | CF_WAIT_BIT | CF_RDY_BIT | CF_OE_BIT | CF_CE2_BIT | CF_CE1_BIT | CF_RST_BIT)
#define PORTF_GPR       0x0 // no secondary functions

// -----------------------------------------------------------------------------------------------
// This is not a GPIO specific register but is used in the multiplexing of a few signals and will be
// needed when trying to get certain alternate functions working.
// Function Multiplexing Control Register Bit Defines.
#define	UART4_RXD_CTL_BIT	    BIT25	            // 0 = USBH1_TXDP(PB29) GPIOs AOUT is input to RXD of UART4. 1 = Alternate signal of USBH1_RXDP(PB31) is input to RXD of UART4.
#define	UART4_RTS_CTL_BIT	    BIT24	            // 0 = USBH1_RXDP (PB31) GPIOs AOUT is input to RTS of UART4. 1 = Alternate signal of USBH1_FS (PB26) is input to RTS of UART4.
#define	KP_COL6_CTL_BIT		    BIT18	            // 0 = Alternate signal of TEST_WB2 (PE0) is input to column 6 of keypad. 1 = Alternate signal of UART2_TXD (PE6) is input to column 6 of keypad.
#define	KP_ROW7_CTL_BIT		    BIT17	            // 0 = Alternate signal of TEST_WB0 (PE2) is input to row 7 of keypad. 1 = Alternate signal of UART2_RTS (PE4) is input to row 7 of keypad.
#define	KP_ROW6_CTL_BIT		    BIT16	            // 0 = Alternate signal of TEST_WB1 (PE1) is input to row 6 of keypad. 1 = Alternate signal of UART2_RXD (PE7) is input to row 6 of keypad.
#define	CRM_SPA_SEL_BIT		    BIT12	            // 1 = Selects Well Bias optimal power savings when Well Bias is enabled. This bit has no effect when Well Bias is disabled.
#define	NF_FMS_BIT			    BIT5	            // 0 = NAND Flash with 512B page size. 1 = NAND Flash with 2 Kbyte page size.
#define	NF_16BIT_SEL_BIT	    BIT4	            // 0 = NAND Flash 8-bit. 1 = NAND Flash 16-bit.
#define	SLCDC_SEL_BIT		    BIT2	            // 0 = On Chip SLCDC drives the SLCDC port. 1 = BB can write directly to the SLCDC port.
#define	SDCS1_SELL_BIT		    BIT1	            // 0 = CS3 selected. 1 = CSD1 selected.
#define	SDCS0_SEL_BIT		    BIT0	            // 0 = CS2 selected. 1 = CSD0 selected.

// Define Function Multiplexing Control Register
#define FMCR_REG		(UART4_RTS_CTL_BIT | KP_COL6_CTL_BIT | KP_ROW7_CTL_BIT | KP_ROW6_CTL_BIT | CRM_SPA_SEL_BIT | SDCS1_SELL_BIT | SDCS0_SEL_BIT)
