/*********************************************************************
 *
 * Copyright:
 *  MOTOROLA, INC. All Rights Reserved.  
 *  You are hereby granted a copyright license to use, modify, and
 *  distribute the SOFTWARE so long as this entire notice is
 *  retained without alteration in any modified and/or redistributed
 *  versions, and that such modified versions are clearly identified
 *  as such. No licenses are granted by implication, estoppel or
 *  otherwise under any patents or trademarks of Motorola, Inc. This 
 *  software is provided on an "AS IS" basis and without warranty.
 *
 *  To the maximum extent permitted by applicable law, MOTOROLA 
 *  DISCLAIMS ALL WARRANTIES WHETHER EXPRESS OR IMPLIED, INCLUDING 
 *  IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR
 *  PURPOSE AND ANY WARRANTY AGAINST INFRINGEMENT WITH REGARD TO THE 
 *  SOFTWARE (INCLUDING ANY MODIFIED VERSIONS THEREOF) AND ANY 
 *  ACCOMPANYING WRITTEN MATERIALS.
 * 
 *  To the maximum extent permitted by applicable law, IN NO EVENT
 *  SHALL MOTOROLA BE LIABLE FOR ANY DAMAGES WHATSOEVER (INCLUDING 
 *  WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS 
 *  INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY
 *  LOSS) ARISING OF THE USE OR INABILITY TO USE THE SOFTWARE.   
 * 
 *  Motorola assumes no responsibility for the maintenance and support
 *  of this software
 ********************************************************************/

/*
 * File:        cpuio.h
 * Purpose:     Function prototypes and memory map information.
 *
 * Notes:
 */

#ifndef _INIT_H
#define _INIT_H

#include "mcf532x.h"

#define INT_SRAM_ADDRESS        (0x80000000)
#define INT_SRAM_SIZE           (0x00008000)

#define CPU_FREQUENCY           240000000
#define BUS_FREQUENCY           80000000

#define LOOPS_PER_USEC			5

#define DEFAULT_BAUD_RATE 38400

#endif /* _INIT_H */

// LCD Defines
//
// The LCD frame buffer is fixed at 0xC0080000, which is .5Mbyte from the
// beginning of SDRAM space.  Note that we access it 16-bits at a time.
//
#define LCD_BUF_ADD				0x41000000
#define LCD_BUF(_x_)			*(vushort *)(LCD_BUF_ADD + _x_)	// Frame Buffer
#define USE_FONT8X8
#define LCD_GET_PIXEL_ADD(_X_, _Y_)	(((_Y_ * PIXELS_PER_ROW) + _X_)*2)

// defines for the display geometry - OSD043TN14 480x272 TFT
#define PIXELS_PER_ROW			480		//
#define PIXELS_PER_COL			272		//
#define BITS_PER_PIXEL			16		//
#define LCD_H_WIDTH 			41		// pulse width in pixels
#define LCD_H_FRONT 			2		// front porch (sync to enable)
#define LCD_H_BACK 				2		// back porch (enable to sync)
#define LCD_V_WIDTH 			10		// pulse width in lines
#define LCD_V_FRONT 			2		// front porch (sync to enable)
#define LCD_V_BACK 				2		// back porch (enable to sync)
#define LCD_PCD					7		// fsys/3 = 80Mhz/PCD +1 = Pixel Clock ~ 10Mhz

#define TOP						0
#define BOTTOM					(PIXELS_PER_COL-1)
#define LEFT					0
#define RIGHT					(PIXELS_PER_ROW-1)
#define CENTER_X				(PIXELS_PER_ROW/2)
#define CENTER_Y				(PIXELS_PER_COL/2)

#define ROWS_PER_SCREEN			34
#define COLS_PER_SCREEN			60

#define LCD_BG_DEF				9
#define LCD_FG_DEF				15

#define LCD_FB_SIZE(_depth_)				(((PIXELS_PER_COL * PIXELS_PER_ROW) * _depth_) / 8)

#define LCD_ROW_SIZE(_depth_)				((PIXELS_PER_ROW * _depth_) / 8)

#define LCD_GET_ADD(_row_, _col_, _depth_)	(((((_row_ * PIXELS_PER_ROW) * FONT_HEIGHT) \
											  + (_col_ * FONT_WIDTH)) \
											  * _depth_) / 8)

