/* Template MicroMonitor configuration file.
 */

/* PLATFORM_NAME:
 * Some string that briefly describes your target.
 */
#define PLATFORM_NAME       "LogicPD 5329 Fire Engine"

/* CPU_NAME:
 * The name of your CPU (i.e. "PowerPC 405", "Coldfire 5272" ,etc...). 
 */
#define CPU_NAME    "Coldfire 5329"

#define RAM_VECTOR_TABLE	0x40000000
#define APPRAMBASE_OVERRIDE 0x40200000
#define DONT_CENTER_MONHEADER
#define CPU_BE

/* Declare a default mac address and default IP address, although
 * they should be changed with the "ether mac" command and the
 * monrc script!
 */
#define DEFAULT_ETHERADD    "00:23:31:60:00:00"
#define DEFAULT_IPADD       "192.168.0.201"

/* define DONT_USE_SRAM such that uMon won't use SRAM for eth BD
 * tx and rx buffers.  This will also disable the cache
 */
#define DONT_USE_SRAM

/* define DONT_USE_CACHE will leave the cache disabled
 */
#define DONT_USE_CACHE

/* LOOPS_PER_SECOND:
 * Approximately the size of a loop that will cause a 1-second delay.
 * This value can be adjusted by using the "sleep -c" command.
 */
#define LOOPS_PER_SECOND	20000

/* PRE_COMMANDLOOP_HOOK():
 * This is a port-specific function that will be called just before
 * MicroMonitor turns over control to the endless command processing
 * loop in start() (start.c).
#define PRE_COMMANDLOOP_HOOK()	func()
 */

/* If a watchdog macro is needed, this is how you do it
 * (using target specific code in the macro of course)...
 */
#define _MCF_SCM_CWSR                     (*(volatile unsigned char *)(0xFC04001B))
#define WATCHDOG_MACRO \
	{ \
		_MCF_SCM_CWSR = 0x55; \
		_MCF_SCM_CWSR = 0xAA; \
	}

/* Flash bank configuration:
 * Basic information needed to configure the flash driver.
 * Fill in port specific values here.
 */
#define FLASH_BANK0_WIDTH       2
#define FLASH_BANK0_BASE_ADDR  	0x00000000
#define FLASH_LARGEST_SECTOR    (64 * 1024)
#define SINGLE_FLASH_DEVICE 	1
#define FLASH_COPY_TO_RAM 		1
/*
 * Protect the first 512K, which is 8*8K sectors + 7*64K sectors.
 */
#define FLASH_PROTECT_RANGE  	"0-14"

/* If there is a need to have the range of protected sectors locked (and
 * the flash device driver supports it), then enable this macro...
#define LOCK_FLASH_PROTECT_RANGE
 */

/* TFS definitions:
 * Values that configure the flash space that is allocated to TFS.
 * Fill in port specific values here.
 */
/*
 * Our flash is the 28F160C3 (bottom) which has 8*8k sectors followed by 31*64K sectors 
 */
#define TFSSPARESIZE    		FLASH_LARGEST_SECTOR
#define TFSSTART        		(FLASH_BANK0_BASE_ADDR + (512 * 1024))
#define TFSEND          		(FLASH_BANK0_BASE_ADDR + (2048 * 1024) - (64 * 1024) - 1)
#define TFSSPARE        		(FLASH_BANK0_BASE_ADDR + (2048 * 1024) - (64 * 1024))
#define TFSSECTORCOUNT			((TFSSPARE-TFSSTART)/(64 * 1024))
#define TFS_EBIN_ELF    		1
#define TFS_VERBOSE_STARTUP		1
#define TFS_ALTDEVTBL_BASE		&alt_tfsdevtbl_base

/* FLASHRAM Parameters (not required):
 * Primarily used for configuring TFS on battery-backed RAM.
 * For a simple (volatile) RAM-based TFS device, use the
 * "tfs ramdev" command.
 *
 * Define a block of RAM space to be used as a TFS-device.
 * This is a block of RAM that TFS is fooled into treating like flash.
 * It essentially provides a RAM-based file-storage area.
 */
#define FLASHRAM_BASE			0x40080000
#ifdef FLASHRAM_BASE
# define FLASHRAM_END			(0x40200000 - 1)
# define FLASHRAM_SECTORSIZE	0x010000
# define FLASHRAM_SPARESIZE		FLASHRAM_SECTORSIZE
# define FLASHRAM_SECTORCOUNT	(((FLASHRAM_END + 1) - FLASHRAM_BASE) / FLASHRAM_SECTORSIZE)
# define FLASHRAM_BANKNUM		1
# define FLASHBANKS				2
#else
# define FLASHBANKS				1
#endif

/* ALLOCSIZE:
 * Specify the size of the memory block (in monitor space) that
 * is to be allocated to malloc in the monitor.  Note that this
 * size can be dynamically increased using the heap command at
 * runtime.
 */
#define ALLOCSIZE 		(128*1024)

/* MONSTACKSIZE:
 * The amount of space allocated to the monitor's stack.
 */
#define MONSTACKSIZE	(8*1024)

/* INCLUDE_XXX Macros:
 * Set/clear the appropriate macros depending on what you want
 * to configure in your system.  The sanity of this list is tested
 * through the inclusion of "inc_check.h" at the bottom of this list...
 * When starting a port, set all but INCLUDE_MALLOC, INCLUDE_SHELLVARS,
 * INCLUDE_MEMCMDS and INCLUDE_XMODEM to zero.  Then in small steps
 * enable the following major features in the following order:
 *
 *	INCLUDE_FLASH to test the flash device drivers.
 *	INCLUDE_TFS* to overlay TFS onto the flash. 
 *  INCLUDE_ETHERNET, INCLUDE_TFTP to turn the ethernet interface.
 *
 * All other INCLUDE_XXX macros can be enabled as needed and for the
 * most part, they're hardware independent.
 * Note that for building a very small footprint, INCLUDE_MALLOC and
 * INCLUDE_SHELLVARS can be disabled.
 */

/*
 * See app port notes, only have FORCE_BSS_INIT on during port
 * development and debugging
 */
//#define FORCE_BSS_INIT			1
								
#define INCLUDE_MALLOC			1
#define INCLUDE_MEMCMDS         1
#define INCLUDE_SHELLVARS		1
#define INCLUDE_XMODEM          1
#define INCLUDE_EDIT            1
#define INCLUDE_DISASSEMBLER    0
#define INCLUDE_UNZIP           1
#define INCLUDE_ETHERNET        1
#define INCLUDE_ICMP			1
#define INCLUDE_TFTP            1
#define INCLUDE_TFS             1
#define INCLUDE_FLASH           1
#define INCLUDE_LINEEDIT        1
#define INCLUDE_DHCPBOOT        1
#define INCLUDE_TFSAPI          1
#define INCLUDE_TFSSYMTBL       0
#define INCLUDE_TFSSCRIPT       1
#define INCLUDE_TFSCLI          1
#define INCLUDE_EE              1
#define INCLUDE_GDB             0
#define INCLUDE_STRACE          1
#define INCLUDE_CAST            0
#define INCLUDE_STRUCT          0
#define INCLUDE_REDIRECT        0
#define INCLUDE_QUICKMEMCPY     1
#define INCLUDE_PROFILER        0
#define INCLUDE_BBC             0
#define INCLUDE_MEMTRACE        1

/* I'm not using STOREMAC as flash can't be written during boot because it is locked so 
 * it's a bit pointless... the mac can be set with the "ether mac" command 
*/
#define INCLUDE_STOREMAC        0
#define INCLUDE_VERBOSEHELP     1
#define INCLUDE_HWTMR	 	    0
#define INCLUDE_PORTCMD	 	    0
#define INCLUDE_USRLVL	 	    0

/* Some fine tuning (if needed)...
 * If these #defines are not in config.h, they default to '1' in
 * various other include files within uMon source; hence, they're
 * really only useful if you need to turn off ('0') the specific
 * facility or block of code.
 */
#define INCLUDE_TFTPSRVR		0
#define INCLUDE_ETHERVERBOSE	1
#define INCLUDE_MONCMD			1

#include "inc_check.h"
