#include "config.h"
#include "stddefs.h"
#include "cpuio.h"
#include "genlib.h"
#include "ether.h"
#include "cli.h"
#include "warmstart.h"
#include "xparameters.h"
#include "xcache_l.h"
#include "xuartlite_l.h"

/* devInit():
 * Nothing to do here.  Uart-lite can't be configured.
 */
void
devInit(int baud)
{
}

/* ConsoleBaudSet():
 * Just return the default (hard-coded) baud rate because we can't
 * change the baud of uart-lite.
 */
int
ConsoleBaudSet(int baud)
{
	return(DEFAULT_BAUD_RATE);
}

/* target_console_empty():
 * Target-specific portion of flush_console() in chario.c.
 * This function returns 1 if there are no characters waiting to
 * be put out on the UART; else return 0 indicating that the UART
 * is still busy outputting characters from its FIFO.
 * In the early stages of a new port this can simply return 1.
 */
int
target_console_empty(void)
{
	return(1);
}

/* target_putchar():
 * When buffer has space available, load the incoming character
 * into the UART.
 */
int
target_putchar(char c)
{
	XUartLite_SendByte(XPAR_RS232_BASEADDR,(u8)c);
    return((int)c);
}

/* target_gotachar():
 * Return 0 if no char is avaialable at UART rcv fifo; else 1.
 * Do NOT pull character out of fifo, just return status. 
 */
int
target_gotachar(void)
{
	if (XUartLite_mIsReceiveEmpty(XPAR_RS232_BASEADDR))
		return(0);
	return(1);
}

/* target_getchar():
 * Assume there is a character in the UART's input buffer and just
 * pull it out and return it.
 */
int 
target_getchar(void)
{
	return((int)XUartLite_RecvByte(XPAR_RS232_BASEADDR));
}

/* intsoff():
 */
ulong
intsoff(void)
{
	return(0);
}

/* intsrestore():
 */
void
intsrestore(ulong msr)
{
}

/* cacheInitForTarget():
 */
void
cacheInitForTarget(void)
{
	/* Enable Caches */
	XCache_EnableICache(0x8c000001);
	icacheInvalidate = (int(*)(char *,int))XCache_InvalidateICache;
}

/* target_reset():
 */
void
target_reset(void)
{
	extern void reset(void);
	reset();
}

/* If any CPU IO wasn't initialized in reset.S, do it here...
 * This just provides a "C-level" IO init opportunity. 
 */
void
initCPUio(void)
{
	/* ADD_CODE_HERE */
}

int boot_base;

/* main():
 * For the ML403 build (as of Aug 2005), ld is not used (see makefile);
 * hence, main() is the first function called after crt0...
 * Here we call mySPInit() just to get the stack pointer inside MonStack[].
 * Then we jump into umon_start(), which is just start() (in start.c),
 * with a name change to avoid conflict with crt0's start() function.
 */
int
main()
{
	extern void start(int);
	extern void mySPInit(void);
	
	mySPInit();
	start(INITIALIZE);
	puts("start returned!\n");
	return(0);
}
