/*
 *  This source code has been made available to you by IBM on an AS-IS
 *  basis.  Anyone receiving this source is licensed under IBM
 *  copyrights to use it in any way he or she deems fit, including
 *  copying it, modifying it, compiling it, and redistributing it either
 *  with or without modifications.  No license under IBM patents or
 *  patent applications is to be implied by the copyright license.
 *
 *  Any user of this software should understand that IBM cannot provide
 *  technical support for this software and will not be responsible for
 *  any consequences resulting from the use of this software.
 *
 *  Any person who transfers this source code or any derivative work
 *  must include the IBM copyright notice, this paragraph, and the
 *  preceding two paragraphs in the transferred software.
 *
 *  COPYRIGHT   I B M   CORPORATION 1995
 *  LICENSED MATERIAL  -  PROGRAM PROPERTY OF I B M
 *
 *  File Name:   entry.s
 *
 *  Function:   Entry code for the 405 ROM Monitor (openbios)running 
 *				on the Walnut board.
 *
 *  Author:      James Burke  
 *
 *  Change Activity-
 *
 *  Date        Description of Change                                       BY
 *  ---------   ---------------------                                       ---
 *  29-Apr-99   Created                                                     JWB
 *  17-Jun-99   Updated to run with caches on                               JWB
 *  23-Jun-99   Set MCOPT1 BRD_PRF bits to '01' for 16 byte PLB burst read
 *  	        prefetch to improve SDRAM performance.			   JWB
 *  09-Jul-99   Added call to ppcDflush() after rom2ram()                   JWB
 *  14-Aug-99   Moved disabling of PCI addr pipelining for pass1 405GP bug. JWB
 *  24-Aug-99   Added pci ethernet functions to user callable functions.    JWB
 *  07-Sep-99   Moved ppcDflush() call in boot_jump(); leave SDRAM address
 *              pipelining enabled (default) in CNTRL1 reg.                 JWB
 *  22-Sep-99   Updated for sdram_init for 32MB SDRAM DIMM now being 
 *              shipped w/ the board                                        JWB
 *  18-Apr-00   Integrated into the MicroMonitor platform.                  ELS
 *  21-Apr-00   Added INCLUDE_ECC to optionally disable ECC and             ELS
 *				INCLUDE_HERALD to optionally include the startup message.
 *
 * General notice:
 * This code is part of a boot-monitor package developed as a generic base
 * platform for embedded system designs.  As such, it is likely to be
 * distributed to various projects beyond the control of the original
 * author.  Please notify the author of any enhancements made or bugs found
 * so that all may benefit from the changes.  In addition, notification back
 * to the author will allow the new user to pick up changes that may have
 * been made by other users after this version of the code was distributed.
 *
 * Author:	Ed Sutter
 * email:   esutter@lucent.com
 * phone:   908-582-2351
 *
 * This reset code is a combination of code written by several developers.
 * The source code for the IBM "Walnut" BSP was used as a reference for this
 * bootup.
 */

#include "config.h"
#include "warmstart.h"
#include "arch_ppc.h"
#include "cpu_403.h"

#define INCLUDE_ECC 0
#define INCLUDE_HERALD 0
#define ONE_BILLION 1000000000
#define I_CACHEABLE_REGIONS 0x80000001
#define D_CACHEABLE_REGIONS 0x80000000
#define FPGA_BRDC 0xf0300004
#define FPGA_BRDS2 0xf0300006
#define EXT_TIMER_INPUT
#define AUTO_MEMORY_CONFIG
/* #define SECOND_BANK */
#define PASS2_405GP

/*
 * IIC Register Offsets       
 */
#define	IIC0_BASE		0xef600500
#define	IICMDBUF		0x00
#define	IICSDBUF		0x02
#define	IICLMADR		0x04
#define	IICHMADR		0x05
#define	IICCNTL			0x06
#define	IICMDCNTL		0x07
#define	IICSTS			0x08
#define	IICEXTSTS		0x09
#define	IICLSADR		0x0A
#define	IICHSADR		0x0B
#define	IICCLKDIV		0x0C
#define	IICINTRMSK		0x0D
#define	IICXFRCNT		0x0E
#define	IICXTCNTLSS		0x0F
#define	IICDIRECTCNTL	0x10

	.extern	MonStack

	.globl	coldstart, reset, warmstart
	.globl	etheraddr, board_info_vector

etheraddr:
	.byte 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff
	.byte 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff
	.byte 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff

board_info_vector:
	.long whereisbinfo


/* coldstart:
 * This is the code that is pointed to by the resetvector at 0xfffffffc.
 * If we get here, then we know that we took some kind of "hard" reset.
 * Some register must be preserved from here until the point where we
 * call start so that we can pass the state to start().  Since we got
 * here, we will load the register with INITIALIZE to indicate the fact
 * that we took a hard reset (or powerup).
 * Note that the register used should be the register that the compiler 
 * would normally populate if calling a function with a single argument.
 * In this case, it is R3.
 */
reset:
coldstart:
	xor		r0,r0,r0			/* Clear R0 */
	addi	r3,r0,INITIALIZE

/* warmstart:
 * This function can be entered either by having previously hit coldstart,
 * or by the monitor source code calling warmstart().  It is assumed to
 * be a function to the C-code, and it is passed a reason for the warmstart.
 * The register which the compiler uses for storage of the calling argument
 * must be saved so that it can be passed to start().
 */
warmstart:
	mr		r15,r3				/* Copy r3 to r15 because between here and */
								/* the call to start(), r3 is used a lot. */
	xor		r0,r0,r0			/* Clear R0 */
	addi    r4,r0,0x0000		/* Clear and set up some registers.  */
	mtsgr   r4
	mtdcwr  r4
	mtesr   r4					/* clear Exception Syndrome Reg */
	mttcr   r4					/* clear Timer Control Reg */
	mtxer   r4					/* clear Fixed-Point Exception Reg */
	addi    r4,r0,0x1000		/* set ME bit (Machine Exceptions) */
	oris    r4,r4,0x0002		/* set CE bit (Critical Exceptions) */
	mtmsr   r4					/* change MSR */
	ori 	r4,r0,0xffff		/* set r4 to 0xFFFFFFFF (status in the */
								/* dbsr is cleared by setting bits to 1) */
	mtdbsr  r4					/* clear/reset the dbsr */

	/* Invalidate i-cache and then enable for defined memory regions. 
	 * Do the same for the d-cache. Be aware that the caches may be disabled
	 * later in miscLib/main.c based on the user selected cache menu options.
	 */
	bl		invalidate_icache
	addis   r4,r0, I_CACHEABLE_REGIONS@h
	ori     r4,r4, I_CACHEABLE_REGIONS@l
	mticcr  r4    
	addis   r4,r0, D_CACHEABLE_REGIONS@h
	ori     r4,r4, D_CACHEABLE_REGIONS@l
	bl      invalidate_dcache
	mtdccr  r4    

	/* Initialize the External Bus Controller for external peripherals */
	bl      ext_busctl_init

	/* Initialize the Control 0 register for UART control. 
	 * Set UART1 for CTS/RTS and set the UART0 and UART1 external
	 * clock enable to use the external serial clock instead of an  
	 * internally derived clock. Set the FPGA control reg for UART1 to
	 * select CTS/RTS.              
	 */
	addis	r3,r0,0x0000		/* set CTS/RTS for UART1 and set ext */
	ori     r3,r3,0x00C0		/* clock for UART0 and UART1 */

	mfdcr   r4,cntrl0			/* read CNTRL0 */
	or      r3,r3,r4			/* read-modify-write */
	mtdcr   cntrl0,r3			/* set CNTRL0 */

	addis   r4,r0,FPGA_BRDC@h
	ori     r4,r4,FPGA_BRDC@l
	lbz     r3,0(r4)			/* get FPGA board control reg */
	eieio
	ori		r3,r3,0x01			/* set UART1 control to select CTS/RTS */
	stb		r3,0(r4)

	/* Initialize the 405's first UART...
	 * Optionally include a startup message.
	 */
	bl      uart0_init
#if INCLUDE_HERALD
	addis	r3,r0,..herald_GP@h
	ori		r3,r3,..herald_GP@l
	bl		put_string
#endif

	/* Initialize IIC Controller */
	bl      iic0_init

	/* We only want to run sdram_init out of a coldstart, so if R15 contains
	 * the INITIALIZE value, we run sdram_init; else we skip over it.
	 */
	addi	r3,r0,INITIALIZE
	cmp     cr0,0,r15,r3
	bne     ..skip_sdraminit

	/* Initialize SDRAM Controller...
	 * Include a message indicating that it will take a while if ECC is 
	 * enabled.
	 */
#if INCLUDE_ECC
	addis	r3,r0,..sdram_start@h
	ori		r3,r3,..sdram_start@l
	bl		put_string
#endif
	bl      sdram_init

..skip_sdraminit:
	/* Clear reservation bit. */
	addis   r4,r0,0x0000
	lwarx   r3,r0,r4			/* get some data/set resv bit
	stwcx.  r3,r0,r4			/* store out and clear resv bit

	/* Flush data cache. */
	bl      ppcDflush

	/* Call calc_speed() to determine the 405GP system clock period, the 
	 * number of nanoseconds per clock of the 405GP SYS_CLOCK.
	 */
	bl      calc_speed

	/* Initialize the Control 1 register for Timer control. 
	 * The CPU timers can be incremented at the CPU clock frequency or 
	 * at the frequency of an external clock supplied to the TMR_CLK 
	 * input. The default register setting of Control 1 reg is to use 
	 * the CPU. The Walnut board supplies the TMR_CLK input with an 
	 * external clock equal to 1/4 of the 405GP SYS_CLOCK. Calculate
	 * timer ticks per second based on which clock used and store in 
	 * the global variable Ticks_per_sec.
	 */
#ifdef EXT_TIMER_INPUT
	addis	r5,r0,0x0080		/* set CPU ext timer clock enable */
								/* to use external timer */

	mfdcr   r4,cntrl1			/* read CNTRL1 */
	or      r5,r5,r4			/* read-modify-write */
	mtdcr   cntrl1,r5			/* set CNTRL1 */

	slwi	r3,r3,2				/* multiply 405GP SYS_CLOCK period  */
								/* (still in r3) by 4 since on   */
								/* Walnut the ext timer clk is  */
								/* 1/4 of the 405GP SYS_CLOCK (ie, */ 
								/* 4 times 405GP Sys_clock_period) */
	addis   r5,r0,ONE_BILLION@h	/* calculate timer ticks per second */
	ori     r5,r5,ONE_BILLION@l	/* by diving 1 billion by the timer */
	divwu   r3,r5,r3			/* clock period stored in r3 */

	mfpvr   r7					/* check PVR */
	rlwinm  r7,r7,0,0x000003FF	/* use bits 22:31 of pvr to determine */
	cmpi    cr0,0,r7,0x0		/* if 405GP pass 1 (PVR=0x40110000) */
	bne     ..timer_done		/* if not pass 1 OK, set Ticks_per_sec */ 
								/* to external timer clock speed in r3 */
	mfdcr   r5,pllmd			/* else determine CPU/PLB ratio   */
								/* via PLL mode reg and mult ticks  */
								/* per sec by the CPU/PLB ratio. This */
								/* is a pass 1 bug that will be fixed  */
								/* in pass 2. */
	rlwinm  r4,r5,15,0x3		/* get PLB to CPU divide bits (PLBDIV) */
	addi    r4,r4,0x1			/* add one to get divider (1,2,3,or 4) */
	mullw   r3,r3,r4			/* mult ticks per sec (still in r3 */
								/* in MHz) by PLBDIV to get the CPU */
								/* timer speed */

#else 
	/* Note that the method below may introduce a small error in timing.
	 * For example, if the 405GP SYS_CLOCK period is 30ns, we divide
	 * 1 billion by 30 to get 33,333,333Mhz. We then multiply this number
	 * by FBKDIV and PLBDIV to get the CPU speed. If FBKDIV=2 and PLBDIV=2,
	 * our calculated CPU speed is (33,333,333 x 2 x 2) = 133,333,332Mhz, 
	 * 1 Mhz lower than what it should be (133,333,333MHz). 
	 * It would be best to hard code Ticks_per_sec rather than try to 
	 * calculate it since in most environments the CPU speed will be known. 
	 * Here we are trying to support multiple CPU speeds.
	 */
	addis   r5,r0,ONE_BILLION@h		/* convert 405GP SYS_CLOCK period   */
	ori     r5,r5,ONE_BILLION@l		/* to MHz by diving 1 billion by  */
	divwu   r3,r5,r3				/* the timer clock period stored in r3 */

	mfdcr   r5,pllmd				/* read PLL Mode reg  */
	
	rlwinm  r4,r5,3,0x7				/* get PLL forward divide bits (FWDDIV) */
	cmpi    cr0,0,r4,0x7
	beq	..bypass_mode				/* if FWDDIV=b'111' then bypass mode */
					 
	rlwinm  r4,r5,7,0xF				/* get PLL feedback divide bits(FBKDIV) */
	mullw   r3,r3,r4				/* mult 405GP speed (still in r3   */
									/* in MHz) by FBKDIV to get PLB speed */  
	rlwinm  r4,r5,15,0x3			/* get PLB to CPU divide bits (PLBDIV) */
	addi    r4,r4,0x1				/* add one to get divider (1,2,3,or 4) */
	mullw   r3,r3,r4				/* mult PLB clock speed (still in r3 */
									/* in MHz) by PLBDIV to get the CPU  */
									/* clock speed */

..bypass_mode:						/* bypass mode; CPU clock period is */
									/* the same as the 405GP SYS_CLOCK */
									/* period - do not need to do anything */
									/* since 405GP clock speed is */
									/* still in r3 in MHz */
#endif
..timer_done:

	/* Check for 405GP pass1 and ensure PCI addr pipelining 
	 * disabled in the CNTRL1 reg.
	 * This will be fixed in pass2.
	 */
	mfpvr   r7	/* check PVR
	rlwinm  r7,r7,0,0x000003FF	/* use bits 22:31 of pvr to determine */
	cmpi    cr0,0,r7,0x0		/* if 405GP pass 1 (PVR=0x40110000) */
	bne     ..ok_cont
	mfdcr   r4,cntrl1			/* read CNTRL1 */
	rlwinm 	r4,r4,0,19,17		/* ensure bit 18 of cntrl1 (PCI addr */
								/* pipeline enable) is off for pass1 */
	mtdcr   cntrl1,r4			/* set CNTRL1 */
..ok_cont:

SPInit:
	/* initialize the stack pointer (Diab/Data compiler needs 2 words) */
	lis     sp, (MonStack+(MONSTACKSIZE-8))@h
	ori     sp, sp, (MonStack+(MONSTACKSIZE-8))@l
	addi 	r7, r0, -8
	and		sp, sp, r7		/* 8-byte aligned (EABI spec) */

#if 0
	/*
	* initialize r2 and r13 according to EABI standard
	* Warning: The monitor cannot use the small data or small const areas.
	*          When an application runs, it takes over R2 and R13 so the
	*          monitor can't use them and allow itself to be called by the
	*          application.  This also means the monitor cannot call the
	*          Diab/Data standard library since those functions do not
	*          avoid these areas.
	*/
	lis     r2, (_SDA2_BASE_)@h
	ori     r2, r2, (_SDA2_BASE_)@l
	lis     r13, (_SDA_BASE_)@h
	ori     r13, r13, (_SDA_BASE_)@l
#endif

GoToC:
	bl		invalidate_icache	/* Cache had been enabled previously, so */
	addi	r4,0, 0x0000		/* we need to shut if off prior to entering */
	mticcr  r4    				/* C-level because the C-startup assumes */
	addi	r4,0, 0x0000		/* it starts with cache off, then it will */
	bl      invalidate_dcache	/* appropriately enbable cache when it is */
	mtdccr  r4    				/* ready (after calling FlashInit()). */

	mr		r3,r15				/* Copy r15 to r3. */
	b		start


/****************************************************************************
 *
 * Function: invalidate_icache
 */
	.text
	.align 2
	.globl  invalidate_icache

invalidate_icache:
	iccci   r0,r0			/* for 405, iccci invalidates the */ 
	blr						/* entire I cache  */

/****************************************************************************
 *
 * Function: invalidate_dcache
 */
	.text
	.align 2
	.globl  invalidate_dcache    

invalidate_dcache:    
	addi    r5,0,0x0000		/* clear GPR 6 */
	addi    r7,r0, 128		/* do loop for # of dcache lines */
							/* NOTE: dccci invalidates both  */
	mtctr   r7				/* ways in the D cache */
..dcloop:
	dccci   0,r5			/* invalidate line */
	addi    r5,r5, 32		/* bump to next line */
	bdnz    ..dcloop
	blr

/****************************************************************************
 *
 * Function:     ext_busctl_init
 * Description:  Initializes the External Bus Controller for the external 
 *		peripherals. IMPORTANT: This code must run from cache
 *		since you can not reliably change a peripheral banks
 *		timing register (pbxap) while running code from that bank.
 *		For ex., since we are running from ROM on bank 0, we can NOT 
 *		execute the code that modifies bank 0 timings from ROM, so
 *		we run it from cache.
 *	Bank 0 - Flash/SRAM 
 *	Bank 1 - NVRAM/RTC
 *	Bank 2 - KYBD/Mouse Controller
 *	Bank 3 - IRDA
 *	Bank 4 - Pinned out to Expansion connector
 *	Bank 5 - Pinned out to Expansion connector
 *	Bank 6 - Pinned out to Expansion connector
 *	Bank 7 - FPGA regs 
 */
	.text
	.align 2
	.globl  ext_busctl_init

ext_busctl_init:

	addis   r4,r0, I_CACHEABLE_REGIONS@h
	ori     r4,r4, I_CACHEABLE_REGIONS@l

	mficcr  r9						/* get iccr value */
	cmp     cr0,0,r9,r4				/* check if caching already enabled */
	beq     ..icache_on				/* if not, */
	mticcr  r4						/* enable caching  */
..icache_on:
	addis   r3,0,ext_busctl_init@h	/* store the address of the  */
	ori     r3,r3,ext_busctl_init@l	/* ext_busctl_init functn in r3 */
	addi    r4,0,11					/* set ctr to 10; used to prefetch */
	mtctr   r4						/* 10 cache lines to fit this function */
									/* in cache (gives us 8x10=80 instrctns) */
..ebcloop:
	icbt    r0,r3					/* prefetch cache line for addr in r3 */
	addi    r3,r3,32				/* move to next cache line */
	bdnz    ..ebcloop				/* continue for 10 cache lines */

/* Delay to ensure all accesses to ROM are complete before changing
 * bank 0 timings. 200usec should be enough.
 *   200,000,000 (cycles/sec) X .000200 (sec) = 0x9C40 cycles
 */
	addis	r3,0,0x0
	ori     r3,r3,0xA000			/* ensure 200usec have passed since reset */
	mtctr   r3
..spinlp:
	bdnz    ..spinlp				/* spin loop */

/* Memory Bank 0 (Flash/SRAM) initialization */
	addi    r4,0,pb0ap
	mtdcr   ebccfga,r4
	addis   r4,0,0x9B01
	ori     r4,r4,0x5480
	mtdcr   ebccfgd,r4

	addi    r4,0,pb0cr
	mtdcr   ebccfga,r4
	addis   r4,0,0xFFF1				/* BAS=0xFFF,BS=0x0(1MB),BU=0x3(R/W), */
	ori     r4,r4,0x8000			/* BW=0x0(8 bits) */
	mtdcr   ebccfgd,r4

/*
 * Memory Bank 1 (NVRAM/RTC) initialization
 */
	addi    r4,0,pb1ap
	mtdcr   ebccfga,r4
	addis   r4,0,0x0281         
	ori     r4,r4,0x5480
	mtdcr   ebccfgd,r4

	addi    r4,0,pb1cr
	mtdcr   ebccfga,r4
	addis   r4,0,0xF001				/* BAS=0xF00,BS=0x0(1MB),BU=0x3(R/W), */
	ori     r4,r4,0x8000			/* BW=0x0(8 bits) */
	mtdcr   ebccfgd,r4

/*
 * Memory Bank 2 (KYBD/Mouse) initialization
 */
	addi    r4,0,pb2ap
	mtdcr   ebccfga,r4
	addis   r4,0,0x0481
	ori     r4,r4,0x5A80
	mtdcr   ebccfgd,r4

	addi    r4,0,pb2cr
	mtdcr   ebccfga,r4
	addis   r4,0,0xF011				/* BAS=0xF01,BS=0x0(1MB),BU=0x3(R/W), */
	ori     r4,r4,0x8000			/* BW=0x0(8 bits) */
	mtdcr   ebccfgd,r4

/*
 * Memory Bank 3 (IRDA) initialization
 */
	addi    r4,0,pb3ap
	mtdcr   ebccfga,r4
	addis   r4,0,0x0181         
	ori     r4,r4,0x5280
	mtdcr   ebccfgd,r4

	addi    r4,0,pb3cr
	mtdcr   ebccfga,r4
	addis   r4,0,0xF021				/* BAS=0xF02,BS=0x0(1MB),BU=0x3(R/W), */
	ori     r4,r4,0x8000			/* BW=0x0(8 bits) */
	mtdcr   ebccfgd,r4

/*
 * Memory Bank 7 (FPGA regs) initialization
 */
	addi    r4,0,pb7ap
	mtdcr   ebccfga,r4
#ifdef PASS2_405GP
	addis   r4,0,0x0181 			/* TWT=3 (RevB/c errate #24) */        
#else
	addis   r4,0,0x0101         	/* TWT=2 */
#endif
	ori     r4,r4,0x5280
	mtdcr   ebccfgd,r4

	addi    r4,0,pb7cr
	mtdcr   ebccfga,r4
	addis   r4,0,0xF031				/* BAS=0xF03,BS=0x0(1MB),BU=0x3(R/W), */
	ori     r4,r4,0x8000			/* BW=0x0(8 bits) */
	mtdcr   ebccfgd,r4

	cmpi    cr0,0,r9,0x0			/* check if I cache was off when we */
	/* started 
	bne     ..ebc_done				/* if it was on, leave on */
	addis   r4,r0,0x0000			/* if it was off, disable */
	mticcr  r4						/* restore iccr */
	isync

..ebc_done:
	blr


/****************************************************************************
 *
 * Function:     ppcDflush
 * Description:  Flush the data cache. This function does NOT turn off the data
 *               cache after the flush. This function makes use of memory
 *               that hopefully should never be in the data cache under normal
 *               circumstances.
 *                       0xFFFFE000-0xFFFFFFFF
 * Input:        none
 * Output:       none
 */
	.text
	.align  2
	.globl  ppcDflush

ppcDflush:
	addis   r9,r0,0x0002		/* set mask for EE and CE msr bits */
	ori     r9,r9,0x8000
	mfmsr   r12					/* save msr */
	andc    r9,r12,r9
	mtmsr   r9					/* disable EE and CE */
	addi    r10,r0,0x0001		/* enable data cache for unused memory */
	mfdccr  r9					/* region 0xF8000000-0xFFFFFFFF via */
	or      r10,r10,r9			/* bit 31 in dccr */
	mtdccr  r10
	addi    r10,r0,128			/* do loop for # of lines */
	addi    r11,r0,4096			/* D cache set size=4K */
	mtctr   r10
	ori     r10,r0,0xE000		/* start at 0xFFFFE000 */
	add     r11,r10,r11			/* add to get to other side of cache line */
..ppcDflush_loop:
	lwz     r3,0(r10)			/* least recently used side */
	lwz     r3,0(r11)			/* the other side */
	dccci   r0,r11				/* invalidate both sides */
	addi    r10,r10,0x0020		/* bump to next line (32 bytes) */
	addi    r11,r11,0x0020		/* bump to next line (32 bytes) */
	bdnz    ..ppcDflush_loop
	sync						/* allow memory access to complete */
	mtdccr  r9					/* restore dccr */
	mtmsr   r12					/* restore msr */
	blr

/****************************************************************************
 *
 * Subroutine: calc_speed
 * Returns the period of the 405GP SYS_CLOCK in nanoseconds. This value can 
 * then be used to calculate the CPU, PLB, OPB, PCI, and timer speeds. 
 * Here we read FPGA_REG_6 to determine whether the on-board clock or an
 * external clock is being used. If the on-board clock is being used, the 
 * 405GP SYS_CLOCK is 33.33Mhz and thus its period is 30ns.
 * If an external clock is being used, then a 41.66Mhz clock is ASSUMED.
 * A 41.66Mhz clock corresponds to a period of 24ns. Of course if 
 * the actual external clock is any thing else but 41.66Mhz, this would be a
 * bad assumption. 
 */
	.text
	.align 	2
	.global calc_speed

calc_speed:
	addis   r5,r0,FPGA_BRDS2@h	/* set offset for FPGA_REG_6 */
	ori		r5,r5,FPGA_BRDS2@l
	lbz     r5,0x0(r5)			/* read to get 405GP clk info */
	andi.   r4,r5,0x20			/* check if ext clock selected */
	beq     ..freq33			/* if not set, then 33.33Mhz (30ns) */
	addi    r3,r0,24			/* else ASSUME ext clk is 41.66Mhz   */
								/* which is a 24ns period.  */
	b       ..end
..freq33:
	addi    r3,r0,30			/* 33.33Mhz => 30ns period  */
..end:
	blr


/****************************************************************************
 *
 * Function:     sdram_init
 * Description:  Configures SDRAM memory banks.
 *		Auto Memory Configuration option reads the SDRAM EEPROM 
 *		via the IIC bus and then configures the SDRAM memory
 *      banks appropriately. If Auto Memory Configuration is
 *		is not used, it is assumed that a 32MB 12x8(2) non-ECC DIMM is
 *		plugged, ie. the DIMM that shipped wih the Eval board.
 */
	.text
	.align 2
	.globl  sdram_init

sdram_init:
	mflr	r31

#ifndef AUTO_MEMORY_CONFIG
	/*
	 * Set MB0CF for bank 0. (0-16MB) Address Mode 4 since 12x8(2)
	 */
	addi    r4,0,mem_mb0cf
	mtdcr   memcfga,r4
	addis   r4,0,0x0004
	ori     r4,r4,6001
	mtdcr   memcfgd,r4

#ifdef SECOND_BANK
	/*
	 * Set MB2CF for bank 2. (16MB-32MB) Address Mode 4 since 12x8(2)
	 */
	addi    r4,0,mem_mb2cf
	mtdcr   memcfga,r4
	addis   r4,0,0x0104
	ori     r4,r4,6001
	mtdcr   memcfgd,r4
#endif
	/*
	 * Take defaults for SDRAM Timing reg, SDTR1.
	 */

	/*
	 * Delay to ensure 200usec have elapsed since reset. Assume worst
	 * case that the core is running 200Mhz:
	 *   200,000,000 (cycles/sec) X .000200 (sec) = 0x9C40 cycles
	 */
	addis   r3,0,0x0000
	ori     r3,r3,0xA000		/* ensure 200usec have passed since reset */
	mtctr   r3
..spinlp2:
	bdnz    ..spinlp2			/* spin loop */

	/*
	 * Set memory controller options reg, MCOPT1.
	 * Set DC_EN to '1' and BRD_PRF to '01' for 16 byte PLB Burst 
	 * read/prefetch.
	 */
	addi    r4,0,mem_mcopt1
	mtdcr   memcfga,r4
	addis   r4,0,0x8080			/* set DC_EN=1 */
	ori     r4,r4,0000
	mtdcr   memcfgd,r4

	/*
	 * Delay to ensure 10msec have elapsed since reset. This is
	 * required for the MPC952 to stabalize. Assume worst
	 * case that the core is running 200Mhz:
	 *   200,000,000 (cycles/sec) X .010 (sec) = 0x1E8480 cycles
	 * This delay should occur before accessing SDRAM.
	 */
	addis   r3,0,0x001E
	ori     r3,r3,0x8480		/* ensure 10msec have passed since reset */
	mtctr   r3
..spinlp3:
	bdnz    ..spinlp3			/* spin loop */



#else     

/*
 * Memory Auto configuration. Reads EEPROM on the SDRAM DIMM via IIC and sets
 * the SDRAM controller appropriately. Handles ECC as well. 
 */
	/*
	 * Read bytes of interest on SDRAM EEPROM via iic_read subroutine.
	 * These values are required to configure the SDRAM controller
	 * properly. Data is stored in regs 4, 5, 6, and 7.
	 */
	addi    r4,0,0x00
	addi    r5,0,0x00
	addi    r3,0,3				/* set byte to read (# ROWS) */
	bl      iic0_read
	ori     r4,r3,0x00			/* store value in R4 */
	addi    r3,0,4				/* set byte to read (# COLS) */
	bl      iic0_read
	rlwinm  r4,r4,8,0xFFFFFFFF	/* make room for byte */
	or      r4,r3,r4			/* store value in R4 */
	addi    r3,0,17				/* set byte to read (# INT BANKS) */
	bl      iic0_read
	rlwinm  r4,r4,8,0xFFFFFFFF	/* make room for byte */
	or      r4,r3,r4			/* store value in R4 */
	addi    r3,0,5				/* set byte to read (# BANKS) */
	bl      iic0_read
	ori     r5,r3,0x00			/* store value in R5 */
	addi    r3,0,6				/* set byte to read (DATA WIDTH) */
	bl      iic0_read
	ori     r6,r3,0x00			/* store value in R6 */
	addi    r3,0,31				/* set byte to read (BANK SIZE) */
	bl      iic0_read
	ori     r7,r3,0x00			/* store value in R7 */

	/* ELS Note:
	 * With the eval board we got from IBM, the content of R4-R7
	 * are as follows at this point...
	 * R4: 0x000c0804, R5: 0x00000001, R6: 0x00000048, R7: 0x00000008
	 */
	/*
	 * Now we have all the data from the SDRAM EEPROM needed in regs 4, 5
	 * 6 and 7. Reg 4 has # rows, # cols, and # internal banks in 3 LSBytes.
	 * Reg 5 has # banks, Reg 6 has data width, and Reg 7 bank size. No more
	 * iic reads are needed. Now the memory controller needs to be set
	 * up properly based on the data from the SDRAM EEPROM.
	 */

	/*
	 * Check r4 values to determine mode. Mode is determined by the
	 * # rows, # cols, and # of internal banks. All three of these
	 * parameters are stored in r4. Mode should be set as follows:
	 *               rows x cols (internal banks)
	 *   mode 1 - 11x9(2), 11X10(2)
	 *   mode 2 - 12x9(4), 12x10(4)
	 *   mode 3 - 13x9(4), 13x10(4), 13x11(4)
	 *   mode 4 - 12x8(2), 12x8(4)
	 *   mode 5 - 11x8(2), 11x8(4)
	 *   mode 6 - 13x8(2), 13x8(4)
	 *   mode 7 - 13x9(2), 13x10(2)           
	 * The mode must be programmed in each of the Memory Configuration
	 * regs (MBxCF) for each enabled bank of SDRAM. Since we only have
	 * one DIMM on the Walnut board, the mode will be the same for
	 * each enabled SDRAM bank.
	 */
	addis   r14,0,0x0B
	ori     r14,r14,0x0902
	cmp     0,0,r4,r14			/* 11x9(2) */
	beq     ..mode1
	addis   r14,0,0x0B
	ori     r14,r14,0x0A02
	cmp     0,0,r4,r14			/* 11x10(2) */
	beq     ..mode1
	addis   r14,0,0x0C
	ori     r14,r14,0x0904
	cmp     0,0,r4,r14			/* 12x9(4) */
	beq     ..mode2
	addis   r14,0,0x0C
	ori     r14,r14,0x0A04
	cmp     0,0,r4,r14			/* 12x10(4) */
	beq     ..mode2
	addis   r14,0,0x0D
	ori     r14,r14,0x0904
	cmp     0,0,r4,r14			/* 13x9(4) */
	beq     ..mode3
	addis   r14,0,0x0D
	ori     r14,r14,0x0A04
	cmp     0,0,r4,r14			/* 13x10(4) */
	beq     ..mode3
	addis   r14,0,0x0D
	ori     r14,r14,0x0B04
	cmp     0,0,r4,r14			/* 13x11(4) */
	beq     ..mode3
	addis   r14,0,0x0C
	ori     r14,r14,0x0804
	cmp     0,0,r4,r14			/* 12x8(4) */
	beq     ..mode4
	addis   r14,0,0x0C
	ori     r14,r14,0x0802
	cmp     0,0,r4,r14			/* 12x8(2) */
	beq     ..mode4
	addis   r14,0,0x0B
	ori     r14,r14,0x0802
	cmp     0,0,r4,r14			/* 11x8(2) */
	beq     ..mode5
	addis   r14,0,0x0B
	ori     r14,r14,0x0804
	cmp     0,0,r4,r14			/* 11x8(4) */
	beq     ..mode5
	addis   r14,0,0x0D
	ori     r14,r14,0x0802
	cmp     0,0,r4,r14			/* 13x8(2) */
	beq     ..mode6
	addis   r14,0,0x0D
	ori     r14,r14,0x0804
	cmp     0,0,r4,r14			/* 13x8(4) */
	beq     ..mode6
	addis   r14,0,0x0D
	ori     r14,r14,0x0902
	cmp     0,0,r4,r14			/* 13x9(2) */
	beq     ..mode7
	addis   r14,0,0x0D
	ori     r14,r14,0x0A02
	cmp     0,0,r4,r14			/* 13x10(2) */
	beq     ..mode7
	/*
	 * If we get here, then mode bytes read are invalid
	 */
	addis   r3,r0,..invalid_dimm_mode@h
	ori     r3,r3,..invalid_dimm_mode@l
	or      r14,r4,r4					/* save r4 in r14 since call */
	bl      put_string					/* to put_string uses r4 */
	or      r3,r14,r14					/* put mode read into r3 */
	bl      prt_word					/* print our mode values read */
..invalid_dimm_mode_spin:
	b       ..invalid_dimm_mode_spin	/* halt - invalid dimm mode */
	/*
	 * Store mode bit values in r4. We will use this value when 
	 * setting the SDx_AM field (bits 16:18 of the MBxCF regs)
	 * for each of the SDRAM banks. 
	 */
..mode7:
	addi    r4,0,0x6
	b       ..mode_set
..mode6:
	addi    r4,0,0x5
	b       ..mode_set
..mode5:
	addi    r4,0,0x4
	b       ..mode_set
..mode4:
	addi    r4,0,0x3
	b       ..mode_set
..mode3:
	addi    r4,0,0x2
	b       ..mode_set
..mode2:
	addi    r4,0,0x1
	b       ..mode_set
..mode1:
	addi    r4,0,0x0
..mode_set:
	/*
	 * Determine the size of each bank by using the Module Bank Density
	 * value stored in r7 as read from the SDRAM DIMM EEPROM. Since 
	 * the value currently in r7 contains the size per 64-bit
	 * bank on the DIMM, we divide this value by 2 (shift right 1 bit)
	 * to determine the size per 32-bit bank.
	 */
	srwi    r7,r7,1			/* calc size per 32 bit bank */
	cmpi    0,0,r7,0x02		/* check r7 for bank size */
	beq     ..meg8			/* and use value to calc bank end addr */
	cmpi    0,0,r7,0x04
	beq     ..meg16
	cmpi    0,0,r7,0x08
	beq     ..meg32
	cmpi    0,0,r7,0x10
	beq     ..meg64 
	cmpi    0,0,r7,0x20
	beq     ..meg128
	addi    r7,0,0x100		/* must be 256M */
	addi    r8,0,0x6		/* set size bits (SDx_SZ) for MBxCF reg */
	b       ..set_mb
..meg8:
	addi    r7,0,0x08         
	addi    r8,0,0x1		/* set size bits (SDx_SZ) for MBxCF reg */
	b       ..set_mb
..meg16:
	addi    r7,0,0x10        
	addi    r8,0,0x2		/* set size bits (SDx_SZ) for MBxCF reg */
	b       ..set_mb
..meg32:
	addi    r7,0,0x20       
	addi    r8,0,0x3		/* set size bits (SDx_SZ) for MBxCF reg */
	b       ..set_mb
..meg64:
	addi    r7,0,0x40      
	addi    r8,0,0x4		/* set size bits (SDx_SZ) for MBxCF reg */
	b       ..set_mb
..meg128:
	addi    r7,0,0x80     
	addi    r8,0,0x5		/* set size bits (SDx_SZ) for MBxCF reg */

..set_mb:
	/*
	 * Set the SDRAM memory configuration regs (MBxCF). To do this we must
	 * first determine the number of 32-bit banks to be configured. Note 
	 * that the number of banks in r5 is the number of 64-bit banks as 
	 * read from the SDRAM DIMM EEPROM. Since the 405GP only supports
	 * 32-bit memory, each 64-bit bank must be configured as 2 separate
	 * 32-bit banks. We also use the address mode stored in r4, the 
	 * size per 32-bit bank stored in r7, and the bank size bits stored
	 * in r8 to configure the SDRAM memory bank regsisters (MBxCF).
	 * Note that Memory starts at address 0x00000000 and
	 * continues contiguously.
	 */
	rlwinm  r4,r4,13,0xFFFFFFFF	/* set SDx_AM (addr mode field bits */
								/* 16:18) to be used when setting */
								/* MBxCF regs */
	rlwinm  r8,r8,17,0xFFFFFFFF	/* set SDx_SZ (size field bits */
								/* 12:14) to be used when setting */
								/* MBxCF regs */
	or 	r4,r4,r8				/* set SDx_AM and SDx_SZ fields - these */
								/* settings are the same for all  */
								/* SDRAM banks since only 1 DIMM */
	ori 	r4,r4,0x1			/* set SDx_BE bit (bank enable)  */
	rlwinm  r7,r7,20,0xFFFFFFFF	/* set up r7 (size per bank) to be used */
								/* for calculating the SDx_BA field */
								/* (base addr field bits 0:9) */
								/* when setting the MBxCF regs */


	/*
	 * Set MB0CF for bank 0. No need to add r7 for bank 0 since base
	 * address for MB0CF is 0x00000000.  
	 */
	addi    r2,0,mem_mb0cf
	mtdcr   memcfga,r2
	mtdcr   memcfgd,r4

	/*
	 * Set MB2CF for bank 2.
	 */
	addi    r2,0,mem_mb2cf
	mtdcr   memcfga,r2
	add	r4,r4,r7		/* set base address for this bank  */
						/* by adding bank size to SDx_SZ field */
	mtdcr   memcfgd,r4

	/*
	 * Check r5 which contains the # of 64-bit banks as read from the 
	 * SDRAM DIMM EEPROM, to determine if SDRAM banks 1 and 3 need
	 * to be configured as well.
	 */
	cmpi    0,0,r5,0x2	/* check # of 64-bit banks in r5 */
						/* if 2 then banks MB2CF and MB3CF  */
						/* also need to be configured */
	bne     ..mb_set

..banks2_3:
	/*
	 * Set MB1CF for bank 1.
	 */
	addi    r2,0,mem_mb1cf
	mtdcr   memcfga,r2
	add     r4,r4,r7	/* set base address for this bank */
						/* by adding bank size to SDx_SZ field */
	mtdcr   memcfgd,r4
	/*
	 * Set MB3CF for bank 3.
	 */
	addi    r2,0,mem_mb3cf
	mtdcr   memcfga,r2
	add     r4,r4,r7	/* set base address for this bank */
						/* by adding bank size to SDx_SZ field */
	mtdcr   memcfgd,r4

..mb_set:
	/*
	 * Take defaults for SDRAM Timing reg, SDTR1.
	 */

	/*
	 * Check bank width value in r6 to determine if ECC is possible 
	 * with the SDRAM being used. The Walnut board provides up to four
	 * 32-bit banks of SDRAM, of which only bank 0 and 
	 * bank 1 can support ECC since a 64-bit DIMM is being used. 
	 * Banks 2 and 3 can still be used as non-ECC memory provided   
	 * ECC correction is disabled for those banks. 
	 * 
	 * NOTE: If ECC SDRAM is being used, ECC correction should be enabled
	 * only after the portion of the ECC SDRAM memory to be used has been
	 * initialized with ECC enabled (MCOPT1 MEMCHK bits). That portion of
	 * memory must be initialized with ECC enabled but with correction
	 * disabled to ensure the correct ECC checkbits get stored properly. 
	 * Since ECC does read-modify-writes, if correction was enabled 
	 * before initilaizing memory with ECC enabled, the checkbits returned
	 * on the read would be garbage and would trigger an ECC error.
	 */
#if INCLUDE_ECC
	cmpi    0,0,r6,0x40		/* Check bank width, if 0x40 then */
	beq     ..no_ecc		/* 64 bit and no ECC, else assume ECC */
	addis   r9,0,0x9080		/* set value to program the MCOPT1  */
	ori     r9,r9,0x0000	/* reg; DC_EN=1, MEMCHK='10' to    */
	b	..cont1				/* enable ECC, and BRD_PRF to '01' */
							/* for 16 byte PLB burst read/prefetch	 */
#endif
..no_ecc:
	addis   r9,0,0x8080		/* set value to program the MCOPT1 */
	ori     r9,r9,0x0000	/* reg; DC_EN=1, MEMCHK='00' to */
							/* disable ECC, and BRD_PRF to '01' */
							/* for 16 byte PLB burst read/prefetch */
..cont1:
	/*
	 * Delay to ensure 200usec have elapsed since reset. Assume worst
	 * case that the core is running 200Mhz:
	 *   200,000,000 (cycles/sec) X .000200 (sec) = 0x9C40 cycles
	 */
	addis   r3,0,0x0000
	ori     r3,r3,0xA000	/* ensure 200usec have passed since reset */
	mtctr   r3
..spinlp2:
	bdnz    ..spinlp2		/* spin loop */

	/*
	 * Set memory controller options reg, MCOPT1 with value in r9.
	 * This enables the SDRAM controller. 
	 */
	addi    r2,0,mem_mcopt1
	mtdcr   memcfga,r2
	mtdcr   memcfgd,r9

	/*
	 * Delay to ensure 10msec have elapsed since reset. This is 
	 * required for the MPC952 to stabalize. Assume worst
	 * case that the core is running 200Mhz:
	 *   200,000,000 (cycles/sec) X .010 (sec) = 0x1E8480 cycles
	 * This delay should occur before accessing SDRAM.
	 */
	addis   r3,0,0x001E
	ori     r3,r3,0x8480	/* ensure 10msec have passed since reset */
	mtctr   r3
..spinlp3:
	bdnz    ..spinlp3		/* spin loop */


	/*
	 * Check value in r9 to see if ECC was enabled in MCOPT1 reg..
	 * Bit 3 of the MCOPT1 MEMCHK field can be used to determine if 
	 * ECC enabled.
	 */
	rlwinm. r9,r9,0,0x10000000	/* Check MEMCHK bit 3 of MCOPT1 */
	beq     ..ecc_disabled

..ecc_enabled:
	/*
	 * Now that SDRAM memory is enabled, initialize its entire
	 * address range by writing it with zeros. This is necessary when
	 * using ECC SDRAM since the memory must be initialized with ECC
	 * enabled to ensure the checkbits get initialized correctly.
	 * This must occur BEFORE enabling ECC correction. Otherwise, since
	 * ECC performs read-modify writes, the checkbits returned on the
	 * read would be garbage and an ECC error flagged. ECC correction
	 * should only be enabled after initializing the ECC memory space
	 * with ECC enabled.
	 */
	add     r4,r4,r7			/* use last MBxCF reg setting still */
								/* in r4 and the bank size still in r7 */
								/* to calculate the total size of */
								/* SDRAM memory; here we add the bank */
								/* size to the last bank base addr */
								/* to determine the amount of SDRAM */
	rlwinm   r4,r4,0,0xFFC00000	/* mask off all but the base addr field */
								/* and this value is the final SDRAM */
								/* address */
	addis   r3,r0,0xFFFF		/* set r3 to start address (used */
	ori     r3,r3,0xFFF8		/* to start at addr 0x00000000) */
	addis   r6,r0,0xFFFF		/* set r6 to start address (used */
	ori     r6,r6,0xFFFC		/* to start at addr 0x00000004) */
	addis   r9,r0,0x0000		/* set r9 to test value */
								/* r4 contains the end address */
	rlwinm  r4,r4,29,0xFFFFFFFF	/* r4 contains the end address */
								/* divide by 8 to get loop count */
	mtctr   r4
	mr	r10,r4	/* set r10 to end address (used below) */
	/*
	 * At this point the data cache should be enabled to ensure 64 bit 
	 * writes done for ECC memory init. It was enabled previously.
	 */
..loopz:
	stwu    r9,0x8(r3)	/* store test value */
	stwu    r9,0x8(r6)	/* store test value */
	bdnz    ..loopz

	sync				/* ensure stores complete before  */
						/* enabling ECC correction */

	/*
	 * With memory now initialized and ECC enabled, enable ECC
	 * correction. Enable in bank 0 and check # of banks in r5 
	 * to determine if bank 1 should be enabled as well.
	 * ECC correction is enabled on a per bank basis via the ECCCF reg 
	 * bits 8:15.
	 */
	addis	r9,0,0x0080		/* set bank 0 correction bit */
	cmpi    0,0,r5,0x2		/* check # of 64-bit banks in r5 */
	bne     ..set_ecccf             
	oris	r9,r9,0x0040	/* set bank 1 correction bit */
..set_ecccf:
	/*
	 * Set ECCCF reg to enable correction.
	 */
	addi    r2,0,mem_ecccf
	mtdcr   memcfga,r2
	mtdcr   memcfgd,r9
	sync

	/*
	 * May call ..whups if we fail. ..whups expects failing addr in r3,
	 * expected value in r4, and actual value in r7. Since we need the
	 * value in r4 for the end addr, move it to r10 before setting r4
	 * to the expected value.
	 */
	mr	r10,r4						/* set r10 to end address (used below) */
	addis   r3,r0,0xFFFF			/* set r3 to start address (used */
	ori     r3,r3,0xFFFC			/* to start at addr 0x00000000) */
	addis	r4,0,0x0				/* set r4 to value expected  */
	rlwinm  r10,r10,30,0xFFFFFFFF	/* r10 contains the end address */
									/* divide by 4 to get loop count */
	mtctr   r10
..chk0loop:
	lwzu    r7,0x4(r3)				/* load test value */
	cmpl    cr0,r7,r4				/* compare read value to written value */
	bne     ..whups					/* branch to error if not the same */
	bdnz    ..chk0loop             

..ecc_disabled:

#endif

	mtlr    r31						/* restore lr */
	blr

..whups:
	/* Move to non volatiles. */
	addi    r14,r3,0x0				/* failing address */
	addi    r15,r4,0x0				/* set value */
	addi    r16,r7,0x0				/* actual value */
	/*
	 * Format and print out failing addresss here......
	 */
	addis   r3,r0,..dfail_string@h
	ori     r3,r3,..dfail_string@l
	bl      put_string
	addis   r3,r0,..fail_addr@h
	ori     r3,r3,..fail_addr@l
	bl      put_string
	addi    r3,r14,0x0				/* get failing address */
	bl      prt_word
	addis   r3,r0,..newline@h
	ori     r3,r3,..newline@l
	bl      put_string
	addis   r3,r0,..fail_testval@h
	ori     r3,r3,..fail_testval@l
	bl      put_string
	addi    r3,r15,0				/* get set value */
	bl      prt_word
	addis   r3,r0,..newline@h
	ori     r3,r3,..newline@l
	bl      put_string
	addis   r3,r0,..fail_actual@h
	ori     r3,r3,..fail_actual@l
	bl      put_string
	addi    r3,r16,0x0				/* get actual */
	bl      prt_word
	addis   r3,r0,..newline@h
	ori     r3,r3,..newline@l
	bl      put_string

..dead:
	b       ..dead

/* Ram check failure string. */

..dfail_string:
	.byte "Data area failed RW test!! "
	.byte 0x0d,0x0a,0x00
	.align 2
..fail_addr:
	.byte "Failing Address:  "
	.align 2
..fail_testval:
	.byte "Expected : "
	.align 2
..fail_actual:
	.byte "Actual   : "
	.align 2
..newline:
	.byte 0x0d,0x0a,0x00
	.align 2

/****************************************************************************
 *
 * Function:     iic0_init
 * Description:  Initializes IIC0 for use of IIC bus during memory
 *               configuration.
 */
	.text
	.align 2
	.globl  iic0_init

iic0_init:
	/*
	 * Initialize IIC0 controller.
	 */
	addis   r3,0,IIC0_BASE@h		/* r3 <-base addr of IIC0 cntrlr */
	ori     r3,r3,IIC0_BASE@l
	addi    r4,0,0x00
	stb     r4,IICLMADR(r3)			/* clear lo master address */
	eieio
	stb     r4,IICHMADR(r3)			/* clear hi master address */
	eieio
	stb     r4,IICLSADR(r3)			/* clear lo slave address */
	eieio
	stb     r4,IICHSADR(r3)			/* clear hi slave address */
	eieio
	addi    r4,0,0x08
	stb     r4,IICSTS(r3)			/* clear status */
	eieio
	addi    r4,0,0x8F
	stb     r4,IICEXTSTS(r3)		/* clear extended status */
	eieio
	addi    r4,0,0x03				/* set clock divisor */
	stb     r4,IICCLKDIV(r3)		/* OPB 30-40Mhz => 3 */
	eieio							/* OPB 40-50Mhz => 4 */
	addi    r4,0,0x00
	stb     r4,IICINTRMSK(r3)		/* no interrupts */
	eieio
	stb     r4,IICXFRCNT(r3)		/* clear transfer count */
	eieio
	addi    r4,0,0xF0
	stb     r4,IICXTCNTLSS(r3)		/* clear extended control & stat */
	eieio
	addi    r4,0,0x43				/* set mode control - flush */
	stb     r4,IICMDCNTL(r3)		/* master data buf, enable hold */
	eieio							/* SCL, exit unknown state */
	addi    r4,0,0x00
	stb     r4,IICCNTL(r3)			/* clear control reg */
	eieio

	blr

/****************************************************************************
 *
 * Function:     iic0_read
 * Description:  Does a one byte read from the SDRAM EEPROM via IIC0 bus.
 *               You must do a one byte write to the device addr with the
 *               device subaddress to be read before doing a read.
 *		The SDRAM EEPROM IIC device address is 0xA1 for read and   
 *               and 0xA0 for write.      
 * Input: r3 = device subaddress of byte to be read from SDRAM EEPROM
 * Output r3 = byte value of device subaddress read
 */
	.text
	.align 2
	.globl  iic0_read

	/*
	 * iic_read routine - does 1 byte read from SDRAM EEPROM on IIC0 bus.
	 * iic combined format read requires a write of the byte we want to
	 * read, before doing the read.
	 */
iic0_read:
	addis   r10,0,IIC0_BASE@h		/* r10 <-base addr of IIC0 cntrlr */
	ori     r10,r10,IIC0_BASE@l 

	addi    r11,0,0x08				/* clear status */
	stb     r11,IICSTS(r10)
	eieio

	addis   r12,r0,0x0001			/* set up counter for timeout */
	mtctr   r12
..chk_sts1:
	lbz     r11,IICSTS(r10)			/* read status */
	eieio
	andi.   r11,r11,0x01			/* check for pending transfer */
	beq     ..sts_ok1				/* if 0, OK */
	bdnz    ..chk_sts1
	/*
	 * If we get here, we timed out on clear status
	 */
	addis   r3,r0,..iic0_timeout_fail@h
	ori     r3,r3,..iic0_timeout_fail@l
	bl      put_string
..iic0_timeout_fail_spin:
	b       ..iic0_timeout_fail_spin/* halt - can not clear status */
..sts_ok1:
	lbz     r11,IICMDCNTL(r10)		/* read mode control */
	eieio
	ori     r11,r11,0x40			/* flush master data buffer */
	stb     r11,IICMDCNTL(r10)
	eieio
	addi    r11,0,0xA0				/* set device address for */
	stb     r11,IICLMADR(r10)		/* SDRAM EEPROM write (0xA0) */
	eieio
	stb     r3,IICMDBUF(r10)		/* set device sub address */
									/* which is the byte # to read */
	addi    r11,0,0x01				/* start write of 1 byte */
	stb     r11,IICCNTL(r10)
	eieio

	addis   r12,r0,0x0001			/* set up counter for timeout */
	mtctr   r12
..chk_sts2:
	lbz     r11,IICSTS(r10)			/* read status */
	eieio
	andi.   r11,r11,0x01			/* check for pending transfer */
	beq     ..write_ok				/* if 0, OK */
	bdnz    ..chk_sts2
	/*
	 * If we get here, we timed out on writing the SDRAM EEPROM
	 * Could be No DIMM. Halt since memory can not be configured
	 * properly without the EEPROM data.
	 */
	addis   r3,r0,..iic0_read_fail@h
	ori     r3,r3,..iic0_read_fail@l
	bl      put_string
..iic0_read_fail_spin:
	b       ..iic0_read_fail_spin	/* halt - read failed */
..write_ok:
	addi    r11,0,0x08				/* clear status */
	stb     r11,IICSTS(r10)
	eieio

	addis   r12,r0,0x0001			/* set up counter for timeout */
	mtctr   r12
..chk_sts3:
	lbz     r11,IICSTS(r10)			/* read status */
	eieio
	andi.   r11,r11,0x01			/* check for pending transfer */
	beq     ..sts_ok2				/* if 0, OK */
	bdnz    ..chk_sts3
	/*
	 * If we get here, we timed out on clear status
	 */
	addis   r3,r0,..iic0_timeout_fail@h
	ori     r3,r3,..iic0_timeout_fail@l
	bl      put_string
	b       ..iic0_timeout_fail_spin/* halt - can not clear status */
..sts_ok2:
	lbz     r11,IICMDCNTL(r10)		/* read mode control */
	eieio
	ori     r11,r11,0x40			/* flush master data buffer */
	stb     r11,IICMDCNTL(r10)
	eieio
	addi    r11,0,0xA1				/* set device address for */
	stb     r11,IICLMADR(r10)		/* SDRAM EEPROM (0xA1) */
	eieio
	addi    r11,0,0x03				/* read of 1 byte */
	stb     r11,IICCNTL(r10)
	eieio

	addis   r12,r0,0x0001			/* set up counter for timeout */
	mtctr   r12
..chk_sts4:
	lbz     r11,IICSTS(r10)			/* read status */
	eieio
	andi.   r11,r11,0x01			/* check for pending transfer */
	beq     ..read_ok				/* if 0, OK */
	bdnz    ..chk_sts4
	/*
	 * If we get here, we timed out on reading the SDRAM EEPROM
	 * Could be No DIMM. Halt since memory can not be configured
	 * properly without the EEPROM data.
	 */
	addis   r3,r0,..iic0_read_fail@h
	ori     r3,r3,..iic0_read_fail@l
	bl      put_string
	b       ..iic0_read_fail_spin	/* halt - read failed */
..read_ok:
	lbz     r3,IICMDBUF(r10)		/* read byte returned from EEPROM */
	eieio

	blr								/* return with data in R3 */

/*****************************************************************************
 *
 * Function: prt_word
 * Take reg 3 and spit it out in hex.
 */
	.global putmsrtest
	.global msrshow
putmsrtest:
	addis   r3,r0,0x1000		/* set ME bit (Machine Exceptions) */
	oris    r3,r3,0x0002		/* set CE bit (Critical Exceptions) */
	mtmsr   r3					/* change MSR */
	blr

msrshow:
	mfmsr	r3

prt_word:
	mflr    r31
	addi    r18,r3,0x0000		/* save in r18 */
	addi    r4,r0,0x8			/* load count in r4 */
	mtctr   r4
..wrdlp:
	rlwinm  r3,r18,4,0xF		/* isolate ls nibble of r18 */
	cmpi    cr0,0,r3,0x9
	addi    r3,r3,0x30
	bgt     ..alpha
	/* A regular number here, add 0x30. */
	b       ..jn
..alpha:
	/* A hex char, add 0x7. */
	addi    r3, r3, 0x7
..jn:
	bl      s1putchar
	rlwinm  r18,r18,4,0xFFFFFFFF
	bdnz    ..wrdlp
	mtlr    r31
	blr


/*****************************************************************************
 *
 * Function: put_string
 * Puts a string pointed to by r3 out on com1.
 */
put_string:
	mflr    r31
	addi    r17,r3,0x0
..pdt_loop:
	lbz     r3,0x0(r17)			/* pull byte */
	cmpi    cr0,0,r3,0x0		/* compare against null string */
	beq     ..dout
	bl      s1putchar			/* output character */
	addi    r17,r17,0x1
	b       ..pdt_loop
..dout:
	mtlr    r31
	blr

#if INCLUDE_HERALD
..herald_GP:
	.byte 0x0d,0x0a
	.byte "405GP Starting up..."
	.byte 0x0d,0x0a,0x00
	.align 2
#endif

#if INCLUDE_ECC
..sdram_start:
	.byte "SDRAM Initialization from EEPROM..."
	.byte "(this takes about a minute!)"
	.byte 0x0d,0x0a,0x00
	.align 2
#endif

/*
 * IIC0 read failure from SDRAM EEPROM.
 */
..iic0_read_fail:
	.byte 0x0d,0x0a,0x0d,0x0a
	.byte "HALTED !!!"
	.byte 0x0d,0x0a
	.byte "IIC0 Read Error from SDRAM EEPROM!"
	.byte 0x0d,0x0a
	.byte "Ensure DIMM is plugged properly!"
	.byte 0x0d,0x0a,0x00
	.align 2
/*
 * IIC0 clear status failure.
 */
..iic0_timeout_fail:
	.byte 0x0d,0x0a,0x0d,0x0a
	.byte "HALTED !!! IIC0 Failure - Timeout due to pending transfer !"
	.byte "                        - Could not clear status reg !"
	.byte 0x0d,0x0a
	.byte "Ensure DIMM is plugged properly!"
	.byte 0x0d,0x0a,0x00
	.align 2
/*
 * Invalid DIMM mode read from EEPROM
 */
..invalid_dimm_mode:
	.byte 0x0d,0x0a,0x0d,0x0a
	.byte "HALTED !!! Invalid DIMM mode read from SDRAM EEPROM!"
	.byte 0x0d,0x0a
	.byte "Ensure DIMM is plugged properly!"
	.byte 0x0d,0x0a
	.byte "Note: byte 1 #rows, byte 2 #cols, byte 3 #internal banks"
	.byte 0x0d,0x0a
	.byte "Value read :  "
	.byte 0x00
	.align 2


/*****************************************************************************
 *
 * Function: uart0_init
 * uart0_init():
 */
#define	UART0_BASE 0xef600300
#define	DATA_REG		0x00    
#define	DL_LSB  	  	0x00    
#define	DL_MSB  		0x01
#define	INT_ENABLE      0x01
#define	FIFO_CONTROL    0x02 
#define	LINE_CONTROL    0x03
#define	MODEM_CONTROL   0x04
#define	LINE_STATUS  	0x05
#define	MODEM_STATUS    0x06
#define	SCRATCH         0x07

	.text
	.align  2
	.globl  async_init
uart0_init:
	addis   r7,r0,UART0_BASE@h		/* set base addr for UART0 */
	ori     r7,r7,UART0_BASE@l
	addi    r3,r0,0x80				/* set DLAB bit */
	stb     r3,LINE_CONTROL(r7)
	addi    r3,r0,0x24				/* set divisors for 19200 baud */
	stb     r3,DL_LSB(r7)
	addi    r3,r0,0x00
	stb     r3,DL_MSB(r7)
	addi    r3,r0,0x03				/* line control 8 bits no parity */
	stb     r3,LINE_CONTROL(r7)
	addi    r3,r0,0x00				/* disable FIFO */
	stb     r3,FIFO_CONTROL(r7)
	addi    r3,r0,0x03				/* modem control DTR RTS */
	stb     r3,MODEM_CONTROL(r7)
	lbz     r4,LINE_STATUS(r7)		/* clear line status */
	lbz     r4,DATA_REG(r7)			/* read receive buffer */
	addi    r3,r0,0x00				/* set scratchpad */
	stb     r3,SCRATCH(r7)
	addi    r3,r0,0x00				/* set interrupt enable reg */
	stb     r3,INT_ENABLE(r7)
	blr

/*
 * Function:     s1putchar
 * Description:  Put a character out on UART0
 */
	.text
	.align  2
	.globl  s1putchar
s1putchar:
	mfmsr   r8
	addi    r9,0,0x7FFF				/* Mask external interrupts */
	oris    r9,r9,0xFFFF
	and     r9,r9,r8
	mtmsr   r9
	addis   r7,r0,UART0_BASE@h		/* set base address for UART0 */
	ori     r7,r7,UART0_BASE@l  
	stb     r3,DATA_REG(r7)			/* put character out */
	eieio
..spnlp:
	lbz     r4,LINE_STATUS(r7)
	eieio
	andi.   r4,r4,0x20				/* check for THRE bit */
	beq     ..spnlp
	mtmsr   r8						/* restore interrupts */
	blr
